/* Structure used to send data to DPLOT via DDE */

/* This is admittedly a bit confusing... Original version of the DPLOT structure used in
   DDE transactions is the first structure below. DPlot's capabilities were later expanded
   (namely, max of 100 curves/plot rather than 20, and larger character strings for the
   legend and curve labels), but it has taken until now (DPlot v1.8) for the DDE structure
   to keep up. I of course don't want to break old programs that use DPlot, so you have
   a choice. Set DPLOT_DDE_VERSION to 2 below to use the old structure. This is equivalent to
   making NO changes to old programs that already use the old structure. Set DPLOT_DDE_VERSION to
   3 below to use the new structure. All your app needs to do is set the Version member
   to DPLOT_DDE_VERSION (which has always been a requirement, so this isn't a change) and 
   you're all set. You *can* use BOTH structures in the same program by renaming one or 
   the other and fiddling around with the function prototypes, but in practice that leads
   to unnecessary confusion.
*/

#define DPLOT_DDE_VERSION 3

#if DPLOT_DDE_VERSION<3

#define MAXC 20

typedef struct tagDPLOT
{
    DWORD Version;           // Caller must set this to DPLOT_DDE_VERSION
    DWORD hwnd;              // handle of client application window
                             // (Use DWORD rather than HWND)
    DWORD DataFormat;        // XY pairs, DX and Y, etc.
    DWORD MaxCurves;         // maximum number of curves (must be <= 20)
                             // = NX for DataFormat = DATA_3D
                             //   ignore for DataFormat = DATA_3DR
    DWORD MaxPoints;         // maximum number of points/curve
                             // = NY for DataFormat = DATA_3D
                             // = 3 * number of points for DATA_3DR
    DWORD NumCurves;         // actual number of curves, always 1 for
                             //   DATA_3D or DATA_3DR
    DWORD Scale;             // scaling code (Linear, Log, etc.)
    float LegendX;           // left coord of legend, expressed as a ratio
                             //    of plot size (0->1)
    float LegendY;           // top coord of legend
    DWORD NP[MAXC];          // actual number of points in each curve;
                             //   cannot exceed MaxPoints.
	                         // For DATA_3DR files, return number of nodes in NP[0]
    DWORD LineType[MAXC];    // line types (see codes below)
    DWORD SymbolType[MAXC];  // symbol types (see codes below)
    DWORD SizeofExtraInfo;   // Extra information following X,Y data
    char  Legend[MAXC+1][40];// Legend[0] is the caption for the legend.
                             // Legend[n] is the legend for the n'th curve.
    char  Label[MAXC][5];    // Strings displayed beside the last data point
                             //   in a curve.
    char  Title[3][80];      // Three title lines.
    char  XAxis[80];         // X Axis label.
    char  YAxis[80];         // Y Axis label.
} DPLOT;

#else

// Version 3 of the structure allows 100 curves and 80 characters for legend strings,
// 40 for curve labels (same limits as DPlot)

#define MAXC 100

typedef struct tagDPLOT
{
    DWORD Version;           // Caller must set this to DPLOT_DDE_VERSION
    DWORD hwnd;              // handle of client application window
                             // (Use DWORD rather than HWND)
    DWORD DataFormat;        // XY pairs, DX and Y, etc.
    DWORD MaxCurves;         // maximum number of curves (must be <= 100)
                             // = NX for DataFormat = DATA_3D
                             //   ignore for DataFormat = DATA_3DR
    DWORD MaxPoints;         // maximum number of points/curve
                             // = NY for DataFormat = DATA_3D
                             // = 3 * number of points for DATA_3DR
    DWORD NumCurves;         // actual number of curves, always 1 for
                             //   DATA_3D or DATA_3DR
    DWORD Scale;             // scaling code (Linear, Log, etc.)
    float LegendX;           // left coord of legend, expressed as a ratio
                             //    of plot size (0->1)
    float LegendY;           // top coord of legend
    DWORD NP[MAXC];          // actual number of points in each curve;
                             //   cannot exceed MaxPoints.
	                         // For DATA_3DR files, return number of nodes in NP[0]
    DWORD LineType[MAXC];    // line types (see codes below)
    DWORD SymbolType[MAXC];  // symbol types (see codes below)
    DWORD SizeofExtraInfo;   // Extra information following X,Y data
    char  Legend[MAXC+1][80];// Legend[0] is the caption for the legend.
                             // Legend[n] is the legend for the n'th curve.
    char  Label[MAXC][40];   // Strings displayed beside the last data point
                             //   in a curve.
    char  Title[3][80];      // Three title lines.
    char  XAxis[80];         // X Axis label.
    char  YAxis[80];         // Y Axis label.
} DPLOT;

#endif

typedef struct tagDPLOT_PLOTMETRICS
{
	DWORD	size;		// Size of this structure; should be set by the caller
	int		hll;		// horizontal and
	int		vll;		//  vertical coordinates of the lower left corner of
						//  the plot, in pixels
	int		hur;		// horizontal and
	int		vur;		//  vertical coordinates of the upper right corner of
						//  the plot, in pixels
	float	xlo;		// value of x at the left plot extent
	float	ylo;		// value of y at the bottom plot extent
	float	xhi;		// value of x at the right plot extent
	float	yhi;		// value of z at the top plot extent
} DPLOT_PLOTMETRICS;

typedef struct tagDPLOT_PLOTMETRICSEX
{
	DWORD	size;		// Size of this structure; should be set by the caller
	int		hll;		// horizontal and
	int		vll;		//  vertical coordinates of the lower left corner of
						//  the plot, in pixels
	int		hur;		// horizontal and
	int		vur;		//  vertical coordinates of the upper right corner of
						//  the plot, in pixels
	float	xlo;		// value of x at the left plot extent
	float	ylo;		// value of y at the bottom plot extent
	float	xhi;		// value of x at the right plot extent
	float	yhi;		// value of z at the top plot extent
	float	stretch[3];	// scale factors for X, Y, and Z
	float	scale;
	float	azimuth;	// view angles
	float	elevation;
} DPLOT_PLOTMETRICSEX;

// Function prototypes for DPLOTLIB.DLL
#if defined(__cplusplus)
extern "C" {
#endif
int __stdcall DPlot_3DBorder(int DocNum, int NumPts, double *array);
int __stdcall DPlot_AddData(int DocNum, int DataType, int NumPts, int Curve, float *array1, float *array2);
int __stdcall DPlot_AddData8(int DocNum, int DataType, int NumPts, int Curve, double *array1, double *array2);
int	__stdcall DPlot_Command(int DocNum, LPSTR commands);
HBITMAP __stdcall DPlot_GetBitmap(int DocNum, int cx, int cy);
HBITMAP __stdcall DPlot_GetBitmapEx(int DocNum, int cx, int cy, DPLOT_PLOTMETRICS *DPM);
HENHMETAFILE __stdcall DPlot_GetEnhMetaFile(int DocNum,float cx, float cy);
int __stdcall DPlot_GetVersion(LPSTR version);
int __stdcall DPlot_MinVersion(int digit1, int digit2, int digit3, int digit4);
int __stdcall DPlot_Plot(DPLOT *DPlot, float *x, float *y, LPSTR commands);
int __stdcall DPlot_Plot8(DPLOT *DPlot, double *x, double *y, LPSTR commands);
int __stdcall DPlot_Plot8p(DPLOT *DPlot, DWORD *px, DWORD *py, LPSTR commands);
HBITMAP __stdcall DPlot_PlotBitmap(DPLOT *DPlot, float *x, float *y, LPSTR commands, int cx, int cy);
HBITMAP __stdcall DPlot_PlotBitmap8(DPLOT *DPlot, double *x, double *y, LPSTR commands, int cx, int cy);
int __stdcall DPlot_PlotToRect(DPLOT *DPlot, float *x, float *y, LPSTR commands, HWND hwnd, RECT *rc);
int __stdcall DPlot_PlotToRect8(DPLOT *DPlot, double *x, double *y, LPSTR commands, HWND hwnd, RECT *rc);
int	__stdcall DPlot_Request(int DocNum, LPSTR item, LPSTR value, DWORD *maxlength);
void __stdcall DPlot_SetErrorMethod(int Method);
HWND __stdcall DPlot_Start(int Minimize, int *WasActive);
void __stdcall DPlot_Stop();
int __stdcall DPlot_ZFromXY(int DocNum, double X, double Y, double *Z);
#if defined(__cplusplus)
}
#endif

// Scaling codes:

#define SCALE_LINEARX_LINEARY     1
#define SCALE_LINEARX_LOGY        2
#define SCALE_LOGX_LINEARY        3
#define SCALE_LOGX_LOGY           4
#define SCALE_TRIPARTITE          5
#define SCALE_LINEARX_PROBABILITY 6
#define SCALE_GRAINSIZE_DIST      7
#define SCALE_POLAR               8
#define SCALE_BARCHART            9
#define SCALE_LOGX_PROBABILITY   10
#define SCALE_PROBX_LINEARY      11
#define SCALE_PROBX_LOGY         12
#define SCALE_PROBX_PROBY        13
#define SCALE_TRIANGLE_PLOT      14
#define SCALE_N185               15

// Unit-specific scaling codes ... combine with above values using OR operator

#define UNITS_DEFAULT                   0
#define UNITS_TRIPARTITE_INCHES      UNITS_DEFAULT   // Velocity (Y) in inches/sec
#define UNITS_TRIPARTITE_FEET        0x00000100L     //                 feet/sec
#define UNITS_TRIPARTITE_MILLIMETERS 0x00000200L     //                 mm/sec
#define UNITS_TRIPARTITE_CENTIMETERS 0x00000300L     //                 cm/sec
#define UNITS_TRIPARTITE_METERS      0x00000400L     //                 meters/sec
#define UNITS_GRAINSIZE_MILLIMETERS  UNITS_DEFAULT   // Grain sizes (X) in mm
#define UNITS_GRAINSIZE_INCHES       0x00000100L     //                    inches
#define UNITS_POLAR_RADIANS          UNITS_DEFAULT   // Rotation (X) in radians
#define UNITS_POLAR_DEGREES          0x00000100L     //                 degrees
#define UNITS_USERDEFINED            0x00007F00L

// Data organization following DPLOT structure:

#define DATA_XYXY         0     // One or more sets of X,Y data
#define DATA_DXY          1     // One or more X,Y curves. Constant spacing in X and same number of points in all curves.
#define DATA_XYYY         2     // One or more X,Y curves. All curves have the same X values.
#define DATA_3D           3     // Z values on a rectangular grid
#define DATA_3DR          4     // Random X,Y,Z values
#define DATA_IMAGE        5		// Used only by DPlot - there's no way to SEND DPlot an image
#define DATA_1D           6     // One or more groups of Y values.
#define DATA_3DS          7     // 3D scatter plot

// Line styles:

#define LINESTYLE_NONE        0
#define LINESTYLE_SOLID       1
#define LINESTYLE_LONGDASH    2
#define LINESTYLE_DOTTED      3
#define LINESTYLE_DASHDOT     4
#define LINESTYLE_MEDDASH     5
#define LINESTYLE_DASHDOTDOT  6
#define LINESTYLE_DASHDOTDOTDOT 7

// Symbol styles
#define SYMBOLSTYLE_NONE         0
#define SYMBOLSTYLE_DOT          1
#define SYMBOLSTYLE_CROSS        2
#define SYMBOLSTYLE_ASTERISK     3
#define SYMBOLSTYLE_X            4
#define SYMBOLSTYLE_SQUARE       5
#define SYMBOLSTYLE_DIAMOND      6
#define SYMBOLSTYLE_TRIANGLE     7
#define SYMBOLSTYLE_OCTAGON      8
#define SYMBOLSTYLE_ITRIANGLE    9
#define SYMBOLSTYLE_HEXAGON     10
#define SYMBOLSTYLE_PENTAGON    11
#define SYMBOLSTYLE_STAR        12
#define SYMBOLSTYLE_FILL        0x00000100L		// May be combined with styles above

#ifdef __GNUC__

/* With GCC we'll resolve DPLOTLIB references at runtime rather than linking to an import library...
   mainly because I can't figure out how to create an import library for GCC that works properly!    */

/* Re-definition of DPlotlib functions */
typedef int __stdcall (*dll_DPlot_3DBorder) (int DocNum, int NumPts, double *array);
typedef int __stdcall (*dll_DPlot_AddData) (int DocNum, int DataType, int NumPts, int Curve, float *array1, float *array2);
typedef int __stdcall (*dll_DPlot_AddData8) (int DocNum, int DataType, int NumPts, int Curve, double *array1, double *array2);
typedef int __stdcall (*dll_DPlot_Command) (int DocNum, LPSTR commands);
typedef int __stdcall (*dll_DPlot_GetBitmap) (int DocNum, int cx, int cy);
typedef int __stdcall (*dll_DPlot_GetBitmapEx) (int DocNum, int cx, int cy, DPLOT_PLOTMETRICS *DPM);
typedef int __stdcall (*dll_DPlot_GetEnhMetaFile) (int DocNum,float cx, float cy);
typedef int __stdcall (*dll_DPlot_GetVersion) (LPSTR version);
typedef int __stdcall (*dll_DPlot_MinVersion) (int digit1, int digit2, int digit3, int digit4);
typedef int __stdcall (*dll_DPlot_Plot) (DPLOT *DPlot, float *x, float *y, LPSTR commands);
typedef int __stdcall (*dll_DPlot_Plot8) (DPLOT *DPlot, double *x, double *y, LPSTR commands);
typedef int __stdcall (*dll_DPlot_Plot8p) (DPLOT *DPlot, DWORD *px, DWORD *py, LPSTR commands);
typedef int __stdcall (*dll_DPlot_PlotBitmap) (DPLOT *DPlot, float *x, float *y, LPSTR commands, int cx, int cy);
typedef int __stdcall (*dll_DPlot_PlotBitmap8) (DPLOT *DPlot, double *x, double *y, LPSTR commands, int cx, int cy);
typedef int __stdcall (*dll_DPlot_PlotToRect) (DPLOT *DPlot, float *x, float *y, LPSTR commands, HWND hwnd, RECT *rc);
typedef int __stdcall (*dll_DPlot_PlotToRect8) (DPLOT *DPlot, double *x, double *y, LPSTR commands, HWND hwnd, RECT *rc);
typedef int __stdcall (*dll_DPlot_Request) (int DocNum, LPSTR item, LPSTR value, DWORD *maxlength);
typedef int __stdcall (*dll_DPlot_SetErrorMethod) (int Method);
typedef int __stdcall (*dll_DPlot_Start) (int Minimize, int *WasActive);
typedef int __stdcall (*dll_DPlot_Stop) (void);
typedef int __stdcall (*dll_DPlot_ZFromXY) (int DocNum, double X, double Y, double *Z);

/* Declaration of DPlotlib functions pointers */
dll_DPlot_3DBorder dPlot_3DBorder;
dll_DPlot_AddData dPlot_AddData;
dll_DPlot_AddData8 dPlot_AddData8;
dll_DPlot_Command dPlot_Command;
dll_DPlot_GetBitmap dPlot_GetBitmap;
dll_DPlot_GetBitmapEx dPlot_GetBitmapEx;
dll_DPlot_GetEnhMetaFile dPlot_GetEnhMetaFile;
dll_DPlot_GetVersion dPlot_GetVersion;
dll_DPlot_MinVersion dPlot_MinVersion;
dll_DPlot_Plot dPlot_Plot;
dll_DPlot_Plot8 dPlot_Plot8;
dll_DPlot_Plot8p dPlot_Plot8p;
dll_DPlot_PlotBitmap dPlot_PlotBitmap;
dll_DPlot_PlotBitmap8 dPlot_PlotBitmap8;
dll_DPlot_PlotToRect dPlot_PlotToRect;
dll_DPlot_PlotToRect8 dPlot_PlotToRect8;
dll_DPlot_Request dPlot_Request;
dll_DPlot_SetErrorMethod dPlot_SetErrorMethod;
dll_DPlot_Start dPlot_Start;
dll_DPlot_Stop dPlot_Stop;
dll_DPlot_ZFromXY dPlot_ZFromXY;

/* Function that loads and initializes DPlotlib functions using DLL implementation */
HINSTANCE dPlot_Init(void) {
	HINSTANCE dplot_dll = NULL;
	
    dplot_dll = LoadLibrary("dplotlib.dll");
    if (dplot_dll) {
		dPlot_3DBorder = (dll_DPlot_3DBorder)GetProcAddress(dplot_dll, "DPlot_3DBorder");
		dPlot_AddData = (dll_DPlot_AddData)GetProcAddress(dplot_dll, "DPlot_AddData");
		dPlot_AddData8 = (dll_DPlot_AddData8)GetProcAddress(dplot_dll, "DPlot_AddData8");
		dPlot_Command = (dll_DPlot_Command)GetProcAddress(dplot_dll, "DPlot_Command");
		dPlot_GetBitmap = (dll_DPlot_GetBitmap)GetProcAddress(dplot_dll, "DPlot_GetBitmap");
		dPlot_GetBitmapEx = (dll_DPlot_GetBitmapEx)GetProcAddress(dplot_dll, "DPlot_GetBitmapEx");
		dPlot_GetEnhMetaFile = (dll_DPlot_GetEnhMetaFile)GetProcAddress(dplot_dll, "DPlot_GetEnhMetaFile");
		dPlot_GetVersion = (dll_DPlot_GetVersion)GetProcAddress(dplot_dll, "DPlot_GetVersion");
		dPlot_MinVersion = (dll_DPlot_MinVersion)GetProcAddress(dplot_dll, "DPlot_MinVersion");
		dPlot_Plot = (dll_DPlot_Plot)GetProcAddress(dplot_dll, "DPlot_Plot");
		dPlot_Plot8 = (dll_DPlot_Plot8)GetProcAddress(dplot_dll, "DPlot_Plot8");
		dPlot_Plot8p = (dll_DPlot_Plot8p)GetProcAddress(dplot_dll, "DPlot_Plot8p");
		dPlot_PlotBitmap = (dll_DPlot_PlotBitmap)GetProcAddress(dplot_dll, "DPlot_PlotBitmap");
		dPlot_PlotBitmap8 = (dll_DPlot_PlotBitmap8)GetProcAddress(dplot_dll, "DPlot_PlotBitmap8");
		dPlot_PlotToRect = (dll_DPlot_PlotToRect)GetProcAddress(dplot_dll, "DPlot_PlotToRect");
		dPlot_PlotToRect8 = (dll_DPlot_PlotToRect8)GetProcAddress(dplot_dll, "DPlot_PlotToRect8");
		dPlot_Request = (dll_DPlot_Request)GetProcAddress(dplot_dll, "DPlot_Request");
		dPlot_SetErrorMethod = (dll_DPlot_SetErrorMethod)GetProcAddress(dplot_dll, "DPlot_SetErrorMethod");
		dPlot_Start = (dll_DPlot_Start)GetProcAddress(dplot_dll, "DPlot_Start");
		dPlot_Stop = (dll_DPlot_Stop)GetProcAddress(dplot_dll, "DPlot_Stop");
		dPlot_ZFromXY = (dll_DPlot_ZFromXY)GetProcAddress(dplot_dll, "DPlot_ZFromXY");
		return(dplot_dll);
    }
    else
		return(NULL);
}
#else

// Not GCC. For all other C compilers we'll link to the appropriate import library DPLOTLIB.LIB.
// The following defines allow us to use the same source for all compilers.

#define dPlot_3DBorder        DPlot_3DBorder
#define dPlot_AddData         DPlot_AddData
#define dPlot_AddData8        DPlot_AddData8
#define dPlot_Command         DPlot_Command
#define dPlot_GetBitmap       DPlot_GetBitmap
#define dPlot_GetBitmapEx     DPlot_GetBitmapEx
#define dPlot_GetEnhMetaFile  DPlot_GetEnhMetaFile
#define dPlot_GetVersion      DPlot_GetVersion
#define dPlot_MinVersion      DPlot_MinVersion
#define dPlot_Plot            DPlot_Plot
#define dPlot_Plot8           DPlot_Plot8
#define dPlot_Plot8p          DPlot_Plot8p
#define dPlot_PlotBitmap      DPlot_PlotBitmap
#define dPlot_PlotBitmap8     DPlot_PlotBitmap8
#define dPlot_PlotToRect      DPlot_PlotToRect
#define dPlot_PlotToRect8     DPlot_PlotToRect8
#define dPlot_Request         DPlot_Request
#define dPlot_SetErrorMethod  DPlot_SetErrorMethod
#define dPlot_Start           DPlot_Start
#define dPlot_Stop            DPlot_Stop
#define dPlot_ZFromXY         DPlot_ZFromXY
#endif
