#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "dplot.h"

// DOUBLEPRECISION is used in this example to switch between single- and double-precision versions of 
// the DPLOTLIB functions. Double-precision requires version 1.96 or later of DPlot. In practice there's
// no good reason to do this, the switch is only included for testing purposes and to illustrate the 
// difference, which is apparent in example 7 if you zoom in. To use single-precision, use 
// "#undef DOUBLEPRECISION"

#define DOUBLEPRECISION
#ifdef DOUBLEPRECISION
#define REAL double
#define DPLOT_PLOT dPlot_Plot8
#else
#define REAL float
#define DPLOT_PLOT dPlot_Plot
#endif

int	ExcelDay(int Month,int Day,int Year);

#define DEGTORAD(x) (x/57.295779513082320876798154814105)

// necessary for LCC:
#ifndef RAND_MAX
#define RAND_MAX 0x7fff
#endif

typedef struct tagXYZ
{
	REAL x;
	REAL y;
	REAL z;
} XYZ;

void main()
{
	DPLOT	DPlot;
	REAL	PI;
	int		DocNum;
	int		option;
	int		i, j;
	int		retval;

#ifdef __GNUC__
	HINSTANCE dll;
    if((dll = dPlot_Init()) == NULL)
	{
		printf("LoadLibrary for DPLOTLIB.DLL failed.\n");
		exit(0);
	}
#endif

	PI = 4.f*(REAL)atan(1.0);
	dPlot_SetErrorMethod(2);

top:
	printf("Enter 0 to exit\n"
		   "      1 for XY plot example\n"
		   "      2 for contour plot of gridded data\n"
		   "      3 for contour plot of randomly-spaced points\n"
		   "      4 for box-and-whisker example\n"
		   "      5 for another XY plot (30 curves)\n"
		   "      6 for 3D scatter plot\n"
		   "      7 for yet another XY plot (world pop. estimate)\n"
		   "      8 to get DPlot/DPlot Jr version number\n"
		   "      9 to get peak values from document 1\n"
		   "     10 for a surface plot with concave border\n"
		   "? ");
	scanf("%d",&option);

	switch(option)
	{
	case 1:
		{
			int		NP;
			char	Metrics[512];
			REAL	*x, *y;
			DWORD	dwSize;

			/* Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
			In this case we've used DATA_XYYY (one X array for one or more Y arrays).
			
			  Since the X values are evenly spaced we could also use
			  DATA_DXY  - X has only 2 elements, DX and X0
			  
				And you can ALWAYS use
				DATA_XYXY - Each curve uses its own X array. This is the only
				option available if the curves have different X values or
				a different number of points.
			*/
			
			NP = 1001;
			x = (REAL *)calloc( NP,   sizeof(REAL));
			y = (REAL *)calloc( NP*2, sizeof(REAL));
			for(i=0; i<NP; i++)
			{
				x[i]    = (4.f*i)/(NP-1);
				y[i]    = (REAL)sin(PI*x[i]);
				y[i+NP] = (REAL)cos(PI*x[i]);
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYYY;
			DPlot.MaxCurves		= 2;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 2;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.05f;
			DPlot.NP[0]			= NP;
			DPlot.NP[1]			= NP;
			DPlot.LineType[0]	= LINESTYLE_SOLID;
			DPlot.LineType[1]	= LINESTYLE_LONGDASH;
			strcpy(DPlot.Legend[1],"sin({\\sp}x)");	// {\s is DPlot-speak for "use symbol font"
			strcpy(DPlot.Legend[2],"cos({\\sp}x)");
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			/* The command string can be as complex as you like, limited to 32K characters.
			As an alternative, if your program will be producing many similar plots then
			you might prefer to create a preferences file, edit the file with a text 
			editor to remove unwanted entries, and use the [GetPreferences("filename")]
			command. 
			*/
			DocNum = DPLOT_PLOT(&DPlot,x,y,"[ManualScale(0,-1.25,4,1.25)][TickInterval(1,0.5,0.25)]"
				"[Caption(\"DPLOTLIB XY Test\")][DocMaximize()]"
				"[ClearEditFlag()]");
			free(x);
			free(y);

  			dwSize = sizeof(Metrics);
  			retval = dPlot_Request(DocNum,"PlotMetrics",Metrics,&dwSize);
  			printf("PlotMetrics returns: %s\n", Metrics);
		}
		break;
		
	case 2:
		{
			int		Nx, Ny;
			REAL	extents[4];
			REAL	xx, yy;
			REAL	*z;
			DWORD	dwSize;
			char	Metrics[512];
			
			/* Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y) */
			
			Nx = 61;	// Columns
			Ny = 41;	// Rows
			z = (REAL *)calloc( Nx*Ny, sizeof(REAL));
			extents[0] = -3.f;		// xlo
			extents[1] = -2.f;		// ylo
			extents[2] =  3.f;		// xhi
			extents[3] =  2.f;		// yhi
			for(i=0; i<Nx; i++)
			{
				xx = extents[0] + ((extents[2]-extents[0])*i)/(Nx-1);
				for(j=0; j<Ny; j++)
				{
					yy = extents[1] + ((extents[3]-extents[1])*j)/(Ny-1);
					z[i*Ny+j] = (REAL)(sin(xx)*cos(yy));
				}
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3D;
			DPlot.MaxCurves		= Nx-1;		// Size of the grid, in grid cells
			DPlot.MaxPoints		= Ny-1;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.Title[1],"z = sin(x)*cos(y)");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			DocNum = DPLOT_PLOT(&DPlot,extents,z,"[Caption(\"DPLOTLIB Contour Test 1\")]"
				"[Contour3D(1)][ContourMethod(0)][ContourGrid(0)][ContourAxes(1)][ContourView(30,20)]"
				"[ContourLevels(20,-1,1)][ContourScales(1,1,1)][ContourLighting(2,0.05)]"
				"[FontPoints(1,10)][FontPoints(6,10)][ZAxisLabel(\"z\")]"
				"[DocMaximize()]");
			free(z);
			dwSize = sizeof(Metrics);
			retval = dPlot_Request(DocNum,"PlotMetrics",Metrics,&dwSize);
			printf("PlotMetrics returns: %s\n", Metrics);
		}
		break;
		
	case 3:
		{
			int		Nz;
			XYZ		*Node;
			
			/* Contour plot with randomly spaced points.
			The random feature of this example is a bit nonsensical since we're using 
			an equation to generate the data. This DPlot feature is much more useful 
			for truly random data and/or data values that aren't organized on a 
			rectangular grid. */
			printf("Enter number of nodes... ");
			fflush(stdin);
			scanf("%d",&Nz);
			Node = (XYZ *)calloc( Nz, sizeof(XYZ));
			for( i=0; i < Nz; i++ ) {
				Node[i].x = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[i].y = -PI + 2*PI * (float)rand()/(float)RAND_MAX;
				Node[i].z = (float)(sin(Node[i].x) * cos(Node[i].y));
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DR;
			DPlot.MaxCurves		= 1;
			DPlot.MaxPoints		= Nz;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.Title[1],"z = sin(x)*cos(y)");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,"[Caption(\"DPLOTLIB Contour Test 2\")]"
				"[Contour3D(0)][ContourGrid(0)][ContourMethod(1)][ContourLevels(21,-1,1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]");
			free(Node);
			printf("return value from DPlot_Plot=%d\n",DocNum);
//			DPlot_Command(DocNum,"[GenerateMesh(20,20,-3,-3,3,3,0,0,4)]"); 

		}
		break;

	case 4:
		{
			int		Ny;
			REAL	*y, *py;
			
			/* Box-and-whisker plot - 3 groups */
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_1D;
			DPlot.MaxCurves		= 3;
			DPlot.MaxPoints		= 65;
			DPlot.NumCurves		= 3;
			DPlot.LegendX		= 0.02f;
			DPlot.LegendY		= 0.7f;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Data sent to DPlot via DPLOTLIB.DLL");
			strcpy(DPlot.YAxis,"Amplitude");
			
			// Number of points in each group is 50 +/- 15:
			Ny = 0;
			for(i=0; i < (int)DPlot.NumCurves; i++)
			{
				DPlot.NP[i] = 50 + (int)(30*(-0.5f + (REAL)rand()/(REAL)RAND_MAX));
				Ny += DPlot.NP[i];
				sprintf(DPlot.Legend[i+1],"Group %c",65+i );
			}
			
			// Generate group data, centered around 5.0 with progressively larger variations
			y = (REAL *)calloc( Ny, sizeof(REAL));
			py = y;
			for( i=0; i < (int)DPlot.NumCurves; i++)
			{
				for(j=0; j < (int)DPlot.NP[i]; j++)
				{
					*py = 5.f + (REAL)(i+1) * (-0.5f + (REAL)rand()/(REAL)RAND_MAX);
					py++;
				}
			}
			
			retval = DPLOT_PLOT(&DPlot,NULL,y,"[Caption(\"DPLOTLIB Box-and-whisker Test\")]"
				"[Stat_PlotType(0)][Stat_GroupInfo(15,4)][Stat_BWOptions(1,1,0,1,1.5,2,3)]"
				"[Stat_GrandMean(3,30)][FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]");
			free(y);
			printf("return value from DPlot_Plot=%d\n",retval);
		}
		break;

	case 5:
		{
			int		NC, NP;
			REAL	*x, *y;
//			char	szCommand[256];
			
			/* Simulate a square wave with a slew of sine waves. First 29 curves are
			sine waves, last curve is the addition of 1st 29.
			
			  Depending on which operating system you're using, attempting this with
			DPLOT_DDE_VERSION=2 will either crash or produce very odd results.    */
			
			NC = 30;
			NP = 1001;
			x = (REAL *)calloc( NP,   sizeof(REAL));
			y = (REAL *)calloc( NP*NC, sizeof(REAL));
			for(i=0; i<NP; i++)
			{
				x[i] = (2*PI*i)/(NP-1);
				y[i+(NC-1)*NP] = 0.f;
				for(j=0; j<NC-1; j++)
				{
					y[i+j*NP] = (REAL)sin((j*2+1)*x[i])/(j*2+1);
					y[i+(NC-1)*NP] += y[i+j*NP];
				}
			}
			
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYYY;
			DPlot.MaxCurves		= NC;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= NC;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.LegendX		= 0.05f;
			DPlot.LegendY		= 0.7f;
			for(j=0; j<NC; j++)
			{
				DPlot.NP[j]	= NP;
				DPlot.LineType[j] = LINESTYLE_SOLID;
  				if(j < NC-1)
  					sprintf(DPlot.Legend[j+1],"sin(%dx)/%d",j*2+1,j*2+1);
			}
  			strcpy(DPlot.Legend[NC],"sin(x)+sin(3x)/3+sin(5x)/5+...");
			strcpy(DPlot.Title[0],"Square wave approximation");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			/* The command string can be as complex as you like, limited to 32K characters.
			As an alternative, if your program will be producing many similar plots then
			you might prefer to create a preferences file, edit the file with a text 
			editor to remove unwanted entries, and use the [GetPreferences("filename")]
			command. 
			*/
			retval = DPLOT_PLOT(&DPlot,x,y,"[ManualScale(0,-1.25,6.5,1.25)][TickInterval(1,0.5,0.25)]"
				"[LineWidth(-2,0)][Caption(\"DPLOTLIB XY Test\")][DocMaximize()]"
				"[ClearEditFlag()]");
			free(x);
			free(y);
			printf("return value from DPlot_Plot=%d\n",retval);
		}
		break;
		
	case 6:
		{
			/* 3D scatter plot - random points on 3 spheres. */

			int		NC, Nz;
			REAL	zz, radius, theta, phi;
			REAL	theta_inc;
			REAL	xmin, xmax, ymin, ymax, zmin, zmax;
			REAL	xx, yy;
			XYZ		*Node;
			char	cmd[512];

			memset(&DPlot,0,sizeof(DPlot));
			Nz = 1000;      // Number of points in each of the NC data sets. Data sets CAN have
							// differing numbers of points. In either case, the Node array is 
							// packed with no unused nodes. For example the first point in the
							// 2nd data set should immediately follow the last point in the 1st
							// data set.
			NC = 3;
			srand( (unsigned)time( NULL ) );
			Node = (XYZ *)calloc( NC*Nz, sizeof(XYZ));
			xmin = xmax = 0.;
			ymin = ymax = 0.;
			zmin = zmax = 0.;
			theta_inc = 2.0*sqrt((double)Nz);
			for(j=0; j<NC; j++)
			{
				xx = (REAL)(-5+10.*rand()/RAND_MAX);
				yy = (REAL)(-5+10.*rand()/RAND_MAX);
				zz = (REAL)(-5+10.*rand()/RAND_MAX);
				radius = (REAL)(4.+4.*rand()/RAND_MAX);
				xmin = min(xmin,xx-radius);
				xmax = max(xmax,xx+radius);
				ymin = min(ymin,yy-radius);
				ymax = max(ymax,yy+radius);
				zmin = min(zmin,zz-radius);
				zmax = max(zmax,zz+radius);
				for( i=j*Nz; i < (j+1)*Nz; i++ ) {
					phi = -PI/2+PI*(double)(i-j*Nz)/(double)Nz;
					theta = theta_inc*PI*(double)(i-j*Nz)/(double)Nz;
					Node[i].x = xx + (REAL)(radius*cos(theta)*cos(phi));
					Node[i].y = yy + (REAL)(radius*sin(theta)*cos(phi));
					Node[i].z = zz + (REAL)(radius*sin(phi));
				}
				sprintf(DPlot.Legend[j+1],"Sphere %d",j+1);
				DPlot.NP[j] = Nz;
			}
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DS;
			DPlot.MaxCurves		= NC;
			DPlot.MaxPoints		= Nz;
			DPlot.NumCurves		= NC;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Legend[0],"Legend");
			strcpy(DPlot.Title[0],"DPLOTLIB - Points on Spheres");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
			
			xmin = (REAL)floor(xmin);
			xmax = (REAL)ceil(xmax);
			ymin = (REAL)floor(ymin);
			ymax = (REAL)ceil(ymax);
			zmin = (REAL)floor(zmin);
			zmax = (REAL)ceil(zmax);

			sprintf(cmd,"[Caption(\"DPLOTLIB Test 6\")]"
				"[TickInterval(1,1,1,1)][ManualScale(%g,%g,%g,%g,%g,%g)][ColorBasis(1)][ContourColorScheme(2)]"
				"[ContourCustomColors(255,65280,16711680)][ContourScales(1,1,1)]"
				"[FontPoints(1,10)][FontPoints(6,10)][DocMaximize()]",
				xmin,ymin,xmax,ymax,zmin,zmax);
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Node,cmd);
			free(Node);
		}
		break;

	case 7:
		{
#define NUMDAYS 10
#define SAMPLESPERDAY (24*60*6)		// 10 second intervals
#define REPLACEMENTRATE 0.013

			int		NP;
			double	JanOne04Pop=6393657063.;
			REAL	BaseDay;
			REAL	CurrentDate;
			time_t	ltime;
			struct	tm *today;
			REAL	elapsed_time;
			REAL	*x, *y;

			BaseDay = (REAL)ExcelDay(1,1,2004);
			time( &ltime );
			today = localtime( &ltime );
			CurrentDate = (REAL)ExcelDay(today->tm_mon+1,today->tm_mday,today->tm_year+1900) +
				(REAL)today->tm_hour/24 + (REAL)today->tm_min/(24*60);

			// We'll estimate the world population every 30 seconds for NUMDAYS days
			NP = NUMDAYS*SAMPLESPERDAY + 1;
			x = (REAL *)calloc( NP, sizeof(REAL));
			y = (REAL *)calloc( NP, sizeof(REAL));
			for(i=0; i<NP; i++)
			{
				x[i] = CurrentDate + (REAL)i/(REAL)SAMPLESPERDAY;
				elapsed_time = x[i] - BaseDay;
				y[i] = (REAL)floor( JanOne04Pop * exp(REPLACEMENTRATE*elapsed_time/365.25) );
			}
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_XYXY;
			DPlot.MaxCurves		= 1;	// Must be >= number of curves we plot
			DPlot.MaxPoints		= NP;	// Anything >= NP will do
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			DPlot.NP[0]			= NP;
			DPlot.LineType[0]	= LINESTYLE_SOLID;
			strcpy(DPlot.Title[0],"World Population Estimate");
			sprintf(DPlot.Title[1],"based on Jan 1, 2004 population of %.10g",JanOne04Pop);
			sprintf(DPlot.Title[2],"and a replacement rate of %g%%",100*REPLACEMENTRATE);
			strcpy(DPlot.XAxis,"Date");
			strcpy(DPlot.YAxis,"Population");
		
			retval = DPLOT_PLOT(&DPlot,x,y,"[Caption(\"DPLOTLIB XY Test\")]"
				"[DateFormat(\"M/dd/yy\")][TimeFormat(\"h:mm:ss\")]"
				"[NumberFormat(0,6)][NumberFormat(1,11)][DocMaximize()][ClearEditFlag()]");
			free(x);
			free(y);
			printf("return value from DPlot_Plot=%d\n",retval);
		}
		break;

	case 8:
		{
			int		result;
			char	version[32];

			result = dPlot_GetVersion(version);
			if(!result)
				printf("DPlot_GetVersion failed!\n");
			else
				printf("%s Version %s\n",(result==1 ? "DPlot" : "DPlot Jr"),version);
		}
		break;

	case 9:
		{
			char	szPeaks[1024];
			DWORD	dwSize;
			
			dwSize = sizeof(szPeaks);
			retval = dPlot_Request(1,"ListPeaks",szPeaks,&dwSize);
			if(retval > 0)
				printf("%s\n",szPeaks);
			else if(retval == -3)
				printf("insufficient buffer size, %d required.\n",dwSize);
			else
				printf("return value of DPlot_Request=%d\n",retval);
		}
		break;

	case 10:
		{
			// surface plot with concave edges
			XYZ		Star[10];	
			double	x0, y0, side;
			x0 = 0;
			y0 = 0;
			side = 10.0;

			Star[0].x = x0 - 0.5*side;
			Star[0].y = y0 - 0.5*side*tan(DEGTORAD(54));
			Star[0].z = 0.0;
			Star[1].x = x0;
			Star[1].y = Star[0].y+0.5*side*tan(DEGTORAD(36));
			Star[1].z = 1.0;
			Star[2].x = x0 + 0.5*side;
			Star[2].y = Star[0].y;
			Star[2].z = 0.0;
			Star[3].x = Star[2].x-0.5*side*(sin(DEGTORAD(18))/cos(DEGTORAD(36)));
			Star[3].y = Star[2].y+0.5*side*(cos(DEGTORAD(18))/cos(DEGTORAD(36)));
			Star[3].z = 1.0;
			Star[4].x = Star[2].x+side*sin(DEGTORAD(18));
			Star[4].y = Star[2].y+side*cos(DEGTORAD(18));
			Star[4].z = 0.0;
			Star[5].x = x0+0.5*side*(sin(DEGTORAD(18))/cos(DEGTORAD(36)));
			Star[5].y = Star[4].y;
			Star[5].z = 1.0;
			Star[6].x = x0;
			Star[6].y = Star[4].y+side*sin(DEGTORAD(36));
			Star[6].z = 0.0;
			Star[7].x = 2*x0 - Star[5].x;
			Star[7].y = Star[5].y;
			Star[7].z = 1.0;
			Star[8].x = 2*x0 - Star[4].x;
			Star[8].y = Star[4].y;
			Star[8].z = 0.0;
			Star[9].x = 2*x0 - Star[3].x;
			Star[9].y = Star[3].y;
			Star[9].z = 1.0;
			memset(&DPlot,0,sizeof(DPlot));
			DPlot.Version		= DPLOT_DDE_VERSION;
			DPlot.DataFormat	= DATA_3DR;
			DPlot.MaxCurves		= 1;
			DPlot.MaxPoints		= 10;
			DPlot.NumCurves		= 1;
			DPlot.Scale			= SCALE_LINEARX_LINEARY;
			strcpy(DPlot.Title[0],"Surface Plot with Concave Edges");
			strcpy(DPlot.Title[1],"See {\\iDefine Boundary} command on {\\iOptions} menu");
			strcpy(DPlot.XAxis,"x");
			strcpy(DPlot.YAxis,"y");
				
			DocNum = DPLOT_PLOT(&DPlot,NULL,(REAL *)Star,"[Caption(\"DPLOTLIB Concave Surface Plot\")]"
					"[Contour3D(0)][ContourMethod(0)][ContourGrid(0)][ContourLevels(20,0,1)]"
					"[DocMaximize()]");
			if(DocNum)
			{
				double border[10][2];
				for(i=0; i<10; i++)
				{
					border[i][0] = Star[i].x;
					border[i][1] = Star[i].y;
				}
				dPlot_3DBorder(DocNum,10,&border[0][0]);
			}
		}
		break;
	
	case 0:
#ifdef __GNUC__
	    FreeLibrary(dll);
#endif
		exit(0);
		
	}
	goto top;	
}

//======================================================================================================
int	ExcelDay(int Month,int Day,int Year)
{
	if(Year < 1900 || Year > 2078)
		return -1;
	else
	{
		if( Month > 2 )
			return (int)(365.25*Year)+(int)(30.6001*(Month+1))+Day-694037;
		else
			return (int)(365.25*(Year-1))+(int)(30.6001*(Month+13))+Day-694037;
	}
}
