using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

/* PLEASE READ: There's a bit of magic under the hood in this demo, so please read these comments before
   revising any of the source or copy/pasting the source into your own application.

   This demo gets a bitmap from DPlot for 1 of the 3 options. It also retrieves scaling information in the
   DPLOT_PLOTMETRICS structure so that we can convert mouse position to plot space and print the mouse
   location as X,Y values. If you don't care about mapping mouse movement, use DPlot_GetBitmap rather than
   DPlot_GetBitmapEx, delete the handling of the MouseMove handler, and you can ignore most of the rest
   of these notes.

   For X and Y scaling information, everything we need to know is returned in the DPLOT_PLOTMETRICS structure
   and we can close the plot using FileClose() after getting the bitmap using DPlot_GetBitmapEx. This is 
   true for XY plots as well as 2D plots of 3D data. However, if we also want Z then we have to get it from 
   DPlot, and the plot *must* remain open. This presents a problem if your application will be producing
   many (more than 32) plots - you'll need to eventually close those plots or you'll generate too many
   document windows in DPlot and get an error when you try to produce a new plot. It's fairly simple in this
   application - we just use FileClose() for the last document before creating a new one. FileClose() won't 
   generate an error for documents that have already been closed, so we could, for example, close the XY plot
   example immediately after getting its bitmap, and close it "again" before creating a new XY plot, w/o error.
   If for whatever reason you need multiple plots open, then you'll need to keep track of the document index
   associated with each plot, and use FileClose() on each of them sooner or later.

   Mouse movement: As mentioned previously, getting scaled X and Y values out of mouse position is handled
   entirely within this application and is very fast. So for only X,Y values we don't need to concern ourselves
   with flooding DPlot with DDE requests, since none are made. But we get Z values from DPlot, not from any
   simple structure that we retrieved from DPlot. There is no way that DDE and DPlot can keep up with the
   frequency of WM_MOUSEMOVE messages this application will get. So if we don't take precautions we'll end up
   with, for example, 20 requests for Z value sent to DPlot before the first one is returned, which gets ugly
   very quickly. If you use [SetErrorMethod(2)] w/o putting on the brakes then you'll be flooded with MessageBox's.
   We handle the problem in this application with the global int "busy". If we're busy, WM_MOUSEMOVE is 
   ignored. We also check to ensure that the mouse has actually moved since the last WM_MOUSEMOVE by storing
   the last mouse location.

   X and Y from mouse movement for 3D views: Cannot be done. Think about it and you'll understand why. DPlot will
   not generate an error message if you call DPlot_GetBitmapEx with a 3D plot viewed in 3D, but the returned
   DPLOT_PLOTMETRICS structure will be completely useless.
*/

namespace test2
{
    public partial class Form1 : Form
    {
        [System.Runtime.InteropServices.DllImport("gdi32.dll")]
        public static extern bool DeleteObject(IntPtr hObject);
        
        double A;
        double StartX;
        double EndX;
        int busy = 0;
        int DocNum = 0;
        int Is3DData = 0;
        int NP;
        int NP3DR;
        int NX3D;
        int NY3D;
        int dxPic, dyPic;
        int WasDPlotActive = 0;
        double xLast = 0;
        double yLast = 0;
        Bitmap myBmp = null;
        CheckState Is3DView;
        dplot.DPLOT_PLOTMETRICSEX DPM;
        Rectangle rcPic;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);
            this.Resize += new EventHandler(Form1_Resize);
            this.pictureBox1.MouseMove += new System.Windows.Forms.MouseEventHandler(this.pictureBox1_MouseMove);

            // XY plot variables
            A      = 0.5;
		    StartX = 0.0;
		    EndX   = 4.0;
		    NP     = 1001;
            textBox1.Text = String.Format("{0}",A);
            textBox2.Text = String.Format("{0}",StartX);
            textBox3.Text = String.Format("{0}",EndX);
            textBox4.Text = String.Format("{0}",NP);
            // 3D variables
            Is3DView = CheckState.Unchecked;
            checkBox1.CheckState = Is3DView;
            // 3D gridded data
            NP3DR = 10000;
            textBox5.Text = String.Format("{0}", NP3DR);
            // 3D variables
            NX3D = NY3D = 50;
            textBox6.Text = String.Format("{0}", NX3D);
            textBox7.Text = String.Format("{0}", NY3D);

            labelCoords.Text = "";
            labelTime.Text = "";

            dplot.DPlot_FindDPLOTLIB();
            dplot.DPlot_SetErrorMethod(2);
            dplot.DPlot_Start(1, ref WasDPlotActive);

            rcPic = pictureBox1.ClientRectangle;
            dxPic = this.Size.Width - (rcPic.Right - rcPic.Left);
            dyPic = this.Size.Height - (rcPic.Bottom - rcPic.Top);
        }
        private void Form1_Resize(object sender, System.EventArgs e)
        {
            if(this.Size.Width > dxPic && this.Size.Height > dyPic)
                pictureBox1.Size = new Size(this.Size.Width-dxPic, this.Size.Height-dyPic);
        }
        void Form1_FormClosing(object Sender, FormClosingEventArgs e)
        {
            if (DocNum != 0) dplot.DPlot_Command(DocNum,"[FileClose()]");
            if (WasDPlotActive == 0) dplot.DPlot_Stop();
        }

        private void PaintMyBitmap()
        {
            IntPtr hbmp;

            if (myBmp != null)
            {
                myBmp.Dispose();
                myBmp = null;
            }
            rcPic = pictureBox1.ClientRectangle;
            DPM.size = Marshal.SizeOf(DPM);
            // DPlot_GetBitmapEx returns a normal Win32 HBITMAP...
            hbmp = dplot.DPlot_GetBitmapEx(DocNum, rcPic.Right - rcPic.Left, rcPic.Bottom - rcPic.Top, ref DPM);
            // ... but we need a .NET Bitmap:
            myBmp = Bitmap.FromHbitmap(hbmp);
            DeleteObject(hbmp);
            pictureBox1.Image = myBmp;
        }

        private void button1_Click(object sender, EventArgs e)
        {
            double dX, Xi, xTick;
            double[] X;
            double[] Y;
            int i;
            string cmds;

            Stopwatch timeoperation = Stopwatch.StartNew();
            timeoperation.Start();

            A = double.Parse(textBox1.Text);
            StartX = double.Parse(textBox2.Text);
            EndX = double.Parse(textBox3.Text);
            NP = int.Parse(textBox4.Text);

            if (NP < 2)
            {
                MessageBox.Show("The number of points must be greater than 1.", "Demo");
                return;
            }
            /* In this example we're using the memory-efficent data format DATA_DXY: 
             * Even spacing in X with the same number of points in all curves. The "X" 
             * array in this case is just 2 elements: the spacing in X and the initial
             * X value. */
            X = new double[2];
            Y = new double[NP*2];

            dX = (EndX - StartX) / (NP - 1);
            X[0] = dX;
            X[1] = StartX;
            for (i = 0; i < NP; i++)
            {
                Xi = StartX + i * dX;
                Y[i] = Math.Sin(A * Math.PI * Xi);
                Y[i + NP] = Math.Cos(A * Math.PI * Xi);
            }
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_DXY;
            dp.MaxCurves = 2;           // Must be >= number of curves we plot 
            dp.MaxPoints = NP;          // Anything >= NP will do 
            dp.NumCurves = 2;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.LegendX = 0.05F;
            dp.LegendY = 0.05F;
            dp.NP[0] = NP;
            dp.NP[1] = NP;
            dp.LineType[0] = dplot.LINESTYLE_SOLID;
            dp.LineType[1] = dplot.LINESTYLE_LONGDASH;
            dp.Title1 = "Data sent to DPlot via DPLOTLIB.DLL";
            dp.XAxis = "x";
            dp.YAxis = "y";
            // Take a stab at a reasonable tick mark interval
            xTick = 1.0;
            while((EndX - StartX) / xTick < 5)
                xTick /= 2;
            while((EndX - StartX) / xTick > 12)
                xTick *= 2;
            /* The command string can be as complex as you like, limited to 32K characters.
            As an alternative, if your program will be producing many similar plots then
            you might prefer to create a preferences file, edit the file with a text 
            editor to remove unwanted entries, and use the [GetPreferences("filename")]
            command. 
            */
            cmds = "[ManualScale(" + 
                String.Format("{0}",StartX) + ",-1.25," + 
                String.Format("{0}",EndX) + ",1.25)]" +
                "[TickInterval(1," + String.Format("{0}",xTick) + ",0.25)]"+
                "[Caption(\"DPLOTLIB XY Plot\")]" +
                "[Legend(1,\"sin(" + (A==1.0 ? "" : String.Format("{0}",A)) + "{\\sp}x)\")]" +
                "[Legend(2,\"cos(" + (A==1.0 ? "" : String.Format("{0}",A)) + "{\\sp}x)\")]" +
                "[Color(1,255,0,0)][Color(2,0,0,255)]" +
				"[TextFont(1, 8,  0,0,0,0,0,\"Arial\")]" +
				"[TextFont(2,12,700,0,0,0,0,\"Arial\")]" +
				"[TextFont(4,10,  0,0,0,0,0,\"Arial\")]" +
				"[TextFont(5,10,  0,0,0,0,0,\"Arial\")]" +
				"[TextFont(6,10,  0,0,0,0,0,\"Arial\")]" +
                "[DocMaximize()][ClearEditFlag()]";
            // Close any open document created by this application:
            if (DocNum != 0) dplot.DPlot_Command(DocNum,"[FileClose()]");
            DocNum = dplot.DPlot_Plot8(ref dp, ref X[0], ref Y[0], cmds);
            if (DocNum != 0)
            {
                PaintMyBitmap();
                Is3DData = 0;
            }
            timeoperation.Stop();
            labelTime.Text = String.Format("Elapsed: {0} msec", timeoperation.ElapsedMilliseconds);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            double[,] Node;
            double dummy=0;
            int i;
            string cmds;
            Random rand;

            Stopwatch timeoperation = Stopwatch.StartNew();
            timeoperation.Start();

            NP3DR = int.Parse(textBox5.Text);
            if (NP3DR < 3)
            {
                MessageBox.Show("The number of points must be 3 or greater.", "Demo");
                return;
            }

            Node = new double[NP3DR,3];
            rand = new Random(1);
            for (i = 0; i < NP3DR; i++)
            {
                Node[i, 0] = -3 + 6 * rand.NextDouble();
                Node[i, 1] = -3 + 6 * rand.NextDouble();
                Node[i, 2] = Math.Sin(Node[i, 0]) * Math.Cos(Node[i, 1]);
            }
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3DR;
            dp.MaxCurves = 1;
            dp.MaxPoints = NP3DR;
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via DPLOTLIB.DLL";
            dp.Title2 = "z = sin(x)*cos(y)";
            dp.XAxis = "x";
            dp.YAxis = "y";
            Is3DView = checkBox1.CheckState;
            cmds = "[Caption(\"DPLOTLIB Surface Plot\")]" +
                (Is3DView == CheckState.Checked ? "[Contour3D(1)][ContourView(30,20)][ContourLighting(2,0.05)]" : "[Contour3D(0)]") +
                "[ContourGrid(0)][ContourAxes(1)]" +
                "[ContourMethod(0)][ContourLevels(20,-1,1)]" +
                "[ContourScales(1,1,1)][ManualScale(-3,-3,3,3,-1,1)]" +
                "[FontPoints(1,8)][FontPoints(2,12)][FontPoints(3,12)]" +
                "[FontPoints(4,10)][FontPoints(5,10)][FontPoints(6,8)][FontPoints(8,10)]" +
                "[ZAxisLabel(\"z\")][DocMaximize()]";
            if (DocNum != 0) dplot.DPlot_Command(DocNum, "[FileClose()]");
            DocNum = dplot.DPlot_Plot8(ref dp, ref dummy, ref Node[0,0], cmds);
            if (DocNum != 0)
            {
                PaintMyBitmap();
                Is3DData = 1;
            }
            timeoperation.Stop();
            labelTime.Text = String.Format("Elapsed: {0} msec", timeoperation.ElapsedMilliseconds);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            double[,] Z;
            double[] extents;
            double dX, dY, X, Y;
            int i, j;
            string cmds;

            Stopwatch timeoperation = Stopwatch.StartNew();
            timeoperation.Start();

            NX3D = int.Parse(textBox6.Text);
            NY3D = int.Parse(textBox7.Text);
            if (NX3D < 1)
            {
                MessageBox.Show("NX must be greater than 0.", "Demo");
                return;
            }
            if (NY3D < 1)
            {
                MessageBox.Show("NY must be greater than 0.", "Demo");
                return;
            }

            Z = new double[NX3D+1, NY3D+1];
            extents = new double[4];
            extents[0] = -3;        // xlo
            extents[1] = -3;        // ylo
            extents[2] = 3;         // xhi
            extents[3] = 3;         // yhi
            dX = (extents[2] - extents[0]) / NX3D;
            dY = (extents[3] - extents[1]) / NY3D;
            for (i = 0; i < NX3D + 1; i++)
            {
                X = extents[0] + i * dX;
                for (j = 0; j < NY3D + 1; j++)
                {
                    Y = extents[1] + j * dY;
                    Z[i, j] = Math.Sin(X) * Math.Cos(Y);
                }
            }
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3D;
            dp.MaxCurves = NX3D;
            dp.MaxPoints = NY3D;
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via DPLOTLIB.DLL";
            dp.Title2 = "z = sin(x)*cos(y)";
            dp.XAxis = "x";
            dp.YAxis = "y";
            Is3DView = checkBox1.CheckState;
            cmds = "[Caption(\"DPLOTLIB Surface Plot\")]" +
                (Is3DView == CheckState.Checked ? "[Contour3D(1)][ContourView(30,20)][ContourLighting(2,0.05)]" : "[Contour3D(0)]") +
                "[ContourGrid(0)][ContourAxes(1)]" +
                "[ContourMethod(0)][ContourLevels(20,-1,1)]" +
                "[ContourScales(1,1,1)][ManualScale(-3,-3,3,3,-1,1)]" +
                "[FontPoints(1,8)][FontPoints(2,12)][FontPoints(3,12)]" +
                "[FontPoints(4,10)][FontPoints(5,10)][FontPoints(6,8)][FontPoints(8,10)]" +
                "[ZAxisLabel(\"z\")][DocMaximize()]";
            if (DocNum != 0) dplot.DPlot_Command(DocNum, "[FileClose()]");
            DocNum = dplot.DPlot_Plot8(ref dp, ref extents[0], ref Z[0, 0], cmds);
            if (DocNum != 0)
            {
                PaintMyBitmap();
                Is3DData = 1;
            }
            timeoperation.Stop();
            labelTime.Text = String.Format("Elapsed: {0} msec", timeoperation.ElapsedMilliseconds);

        }
        private void pictureBox1_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
        {
            double x = e.X;
            double y = e.Y;
            double worldx, worldy;
            double worldz=0;
            int ret;

            if (busy != 0) return;
            if (Is3DData != 0 && Is3DView != 0)
            {
                labelCoords.Text = "";
                return;
            }
            /* This check is necessary because contrary to the function name, this 
             * routine is called whenever the mouse is over the control regardless 
             * of whether it moves. If we didn't do this check then a) we'd update
             * labelCoords much more often than needed and b) we'd generate a flood
             * of calls to DPlot_ZFromXY for the 3D case. */
            if (xLast == x && yLast == y) return;
            xLast = x;
            yLast = y;

            if (x < DPM.hll || x > DPM.hur || y < DPM.vur || y > DPM.vll)
                labelCoords.Text = "";
            else
            {
                busy = 1;
                /* This is only valid for linear scaling (which this demo uses).
                 * For other scaling types see the DPLOTLIB documentation. */
                worldx = DPM.xlo + (x - DPM.hll) * (DPM.xhi - DPM.xlo) / (DPM.hur - DPM.hll);
                worldy = DPM.ylo + (y - DPM.vll) * (DPM.yhi - DPM.ylo) / (DPM.vur - DPM.vll);
                labelCoords.Text = String.Format("X={0,4:F}\nY={1,4:F}", worldx, worldy);
                if (Is3DData != 0)
                {
                    ret = dplot.DPlot_ZFromXY(DocNum, worldx, worldy, ref worldz);
                    if (ret == 1)
                    {
                        labelCoords.Text = labelCoords.Text +
                            String.Format("\nZ={0,4:F}", worldz);
                    }
                }
                busy = 0;
            }
        }
    }
}