using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Timers;
using System.Windows.Forms;

namespace test3
{
    public partial class Form1 : Form
    {
        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool BringWindowToTop(IntPtr hwnd);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool IsIconic(IntPtr hwnd);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool IsWindow(IntPtr hwnd);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool IsZoomed(IntPtr hwnd);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool MoveWindow(IntPtr hwnd, int X, int Y, int nWidth, int nHeight, bool Repaint);

        [System.Runtime.InteropServices.DllImport("user32.dll")]
        public static extern bool ShowWindow(IntPtr hwnd, int nCmdShow);

        [System.Runtime.InteropServices.DllImport("kernel32.dll")]
        public static extern int GetTickCount();

        static System.Windows.Forms.Timer myTimer = new System.Windows.Forms.Timer();

        double TimeStep;
        IntPtr hwndDPlot;
        int DocNum;
        CheckState DrawMethod;
        int StartTicks;
        int StopTicks;
        int Running;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            int WasDPlotActive = 0;
            StopTicks = 0;
            Running = 0;

            // Initial time step = 0.1 seconds
            TimeStep = 0.1;
            textBox1.Text = String.Format("{0}", TimeStep);

            DrawMethod = CheckState.Unchecked;			// Use DPlot_Command
            checkBox1.CheckState = DrawMethod;

            // Initial plot window
            textBox2.Text = "0.,-1.5,60.,1.5";

            // Hide time output while simulation isn't running
            labelTime.Text = "";

            // Set form position to upper left corner of work area.
            Rectangle rcWork = SystemInformation.WorkingArea;
            this.Location = new Point (rcWork.Left,rcWork.Top);

            dplot.DPlot_FindDPLOTLIB();
            dplot.DPlot_SetErrorMethod(2);
            hwndDPlot = dplot.DPlot_Start(0, ref WasDPlotActive);

            /* Move DPlot to upper right corner of screen, sized so that both our form
             * and DPlot are completely visible */
            int cx = rcWork.Right - rcWork.Left - this.Width;
            int cy = rcWork.Bottom - rcWork.Top;
            if (IsZoomed(hwndDPlot))
                ShowWindow(hwndDPlot, 9);
            MoveWindow(hwndDPlot, this.Width, rcWork.Top, cx, cy, true);

            myTimer.Tick += new EventHandler(Timer_Proc);
        }

        private void Timer_Proc(Object myObject,EventArgs myEventArgs)
        {
            double x, y1, y2;
	        string cmds;

            x = (double)(GetTickCount()-StartTicks)/1000.0;
            y1 = Math.Sin(x/4);
            y2 = Math.Cos(x/4);
   
            /* The most efficient method for sending data to DPlot is highly dependent
             * on how many points are sent at once and how many curves are updated at
             * once. For this demo it's a wash; they're roughly equivalent. The
             * advantage of using DPlot_Command with an XY or XYXY command is that
             * multiple curves may be updated in a single call. The advantage of using
             * DPlot_AddData is that the data is sent in its native binary format, so
             * there's no need for translation to character string and back to binary.
             * For a large number of points this advantage becomes much more important.
             * DPlot_AddData also allows you to send an arbitrarily large number of
             * values; DPlot_Command is limited by the maximum command string length
             * (32768 characters). */

            if(DrawMethod == CheckState.Checked)
            {
                dplot.DPlot_AddData8(DocNum, dplot.DATA_XYXY, 1, 1, ref x, ref y1);
                dplot.DPlot_AddData8(DocNum, dplot.DATA_XYXY, 1, 2, ref x, ref y2);
            }
            else
            {
                cmds = String.Format("[SelectCurve(1)][XY({0},{1})][SelectCurve(2)][XY({0},{2})]",x,y1,y2);
        		dplot.DPlot_Command(DocNum,cmds);
            }
            labelTime.Text = String.Format("Time: {0}",x);
        }

        private void StopSimulation()
        {
            StopTicks = GetTickCount();
            myTimer.Stop();
            Running = 0;
            // Redraw the plot and re-enable menu updates
            dplot.DPlot_Command(DocNum, "[ViewRedraw()][MenuUpdates(1)]");
            labelTime.Text = "";
            button1.Text = "&Start";
        }

        private void button1_Click(object sender, EventArgs e)
        {
            /* With a very short time step, we may end up here twice in rapid succession, 
             * and user will have a hard time stopping the demo. This gives us a
             * governor: */
            if (GetTickCount() - StopTicks < 1000) return;

            // If simulation is already running, stop it
			if(Running != 0)
			{
				StopSimulation();
				return;
			}

			// Get timestep
			TimeStep = double.Parse(textBox1.Text);

			// Drawing method (see comments in Timer_Proc)
			DrawMethod = checkBox1.CheckState;
    
			// Change button caption to indicate that clicking it again stops the simulation
            button1.Text = "&Stop";

			/* If DPlot is already running, bring it to the top in case user might have
             * minimized it. */
			if(IsWindow(hwndDPlot)) {
				if(IsIconic(hwndDPlot))
					ShowWindow(hwndDPlot,9);
				else
					BringWindowToTop(hwndDPlot);
			}

			/* Create an empty document window in DPlot and get the document index, which
             * will be used for all subsequent DPlot_Command command strings. */
			DocNum = dplot.DPlot_Command(0, "[FileNew()]");

			/* Instruct DPLOT to allocate space for 2 curves, 1 hour's
             * worth of data: */
            string cmds = String.Format("[FileArrays(2,{0})]",(int)(3600/TimeStep-0.5)+1);
			dplot.DPlot_Command(DocNum,cmds);

			/* Use thick (.03") lines to push DPlot's performance limits a bit, and
             * set a few other plot parameters. */
			dplot.DPlot_Command(DocNum,"[LineWidth(1,30)][LineWidth(2,30)]" +
				"[LineType(1,1)][LineType(2,1)]" +
				"[Color(1,255,0,0)][Color(2,0,0,255)]");

			// Set plot window
            dplot.DPlot_Command(DocNum,"[ManualScale(" + textBox2.Text + ")][AutoSize()]");

			// Set labels
			dplot.DPlot_Command(DocNum,"[XAxisLabel(\"Time (seconds)\")]" +
				"[LegendParams(0.5,10,1,1,.05,.05)]" +
				"[FontPoints(1,8)][FontPoints(2,12)][FontPoints(4,12)][FontPoints(6,10)]" +
				"[Legend(1,\"SIN(t/4)\")]" +
				"[Legend(2,\"COS(t/4)\")]" +
				"[Caption(\"C# Demo\")]");
			if(DrawMethod == CheckState.Checked)
				dplot.DPlot_Command(DocNum,"[Title1(\"RealTime Demo using DPlot_AddData\")]");
			else
				dplot.DPlot_Command(DocNum,"[Title1(\"RealTime Demo using DPlot_Command\")]");

			/* Tell DPlot not to update its menu (which normally would be done every
             * time [SelectCurve()] is used). This isn't strictly necessary, but
             * avoids annoying flashing from the menu and toolbar being continually
             * redrawn, and is a bit more efficient since, for the purposes of this
             * demo, the menu doesn't really need to be updated anyway. HOWEVER,
             * this setting is persistent. If you turn off menu updates then you
             * must eventually turn them back on. In the case of this example this
             * is done when the user stops the simulation and StopSimulation is called. */
			dplot.DPlot_Command(DocNum, "[MenuUpdates(0)]");

			// Get start time from API function GetTickCount
			StartTicks = GetTickCount();

			// Start the timer. See Timer_Proc for sending our values to DPlot
            Running = 1;
            myTimer.Interval = (int)(TimeStep * 1000);
            myTimer.Start();
        }
    }
}