using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

class dplot
{
    public const int DPLOT_DDE_VERSION = 3;   //Version 3 structure allows up to 100 data sets per plot

    [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi)]
    public struct DPLOT
    {
        public int Version;        // version number of this structure, should be set by caller to DPLOT_DDE_VERSION
        public int hwnd;           // handle of client application window
        public int DataFormat;     // XY pairs, DX and Y, etc. See DATA_ constants below
        public int MaxCurves;      /* Dependent on DataFormat
                                        * DataFormat    Description
                                        * DATA_XYXY     Max. number of curves, <= 100
                                        * DATA_DXY      ..
                                        * DATA_XYYY     ..
                                        * DATA_3D       Number of grid cells in X direction,
                                        *               = number of data columns-1
                                        * DATA_3DR      not used
                                        * DATA_1D       Max. number of groups, <= 100
                                        * DATA_3DS      Max. number of data sets, <= 100 */
        public int MaxPoints;      /* DATA_XYXY     Max. number of points/curve
                                        * DATA_DXY      ..
                                        * DATA_XYYY     ..
                                        * DATA_3D       Number of grid cells in Y direction,
                                        *               = number of data rows-1
                                        * DATA_3DR      Number of X,Y,Z triplets
                                        * DATA_1D       Max. number of points/group
                                        * DATA_3DS      Max. number of points/data set */
        public int NumCurves;      // Actual number of curves/groups, always 1 for DATA_3D or DATA_3DR
        public int ScaleCode;      // Scaling code for XY plots (Linear, Log, etc.). See below.
        public float LegendX;      /* Left coord of legend, expressed as a ratio
                                        * of plot size (0->1) (CAN be negative and/or > 1,
                                        * but exact placement is then a bit hard to predict) */
        public float LegendY;      /* Top coord of legend, again expressed as a ratio
                                        * of plot size */
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 100)]
        public int[] NP;
        /* Actual number of points in each curve/group. Not used for DATA_3D.
         * Cannot exceed MaxPoints. */
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 100)]
        public int[] LineType;
        // Line types (see codes below)
        [MarshalAs(UnmanagedType.ByValArray, SizeConst = 100)]
        public int[] SymbolType;
        // Symbol types (see codes below)
        public int SizeofExtraInfo;
        /* Extra information following X,Y data. No need to
         * fill in this member for DPLOTLIB calls */
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 8080)]
        public string Legend;
        /* Legend(0->79) is the caption for the legend
         * Legend(n*80->n*80+79) is the caption for the n'th curve
         * Ideally this would be dimensioned as Legend[101][80],
         * but I don't see a way to do that. Same problem exists for 
         * the Label array (next).
         * Fortunately we can get around this mess with individual 
         * DPlot commands. */
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 4000)]
        public string Label;
        /* Strings displayed beside the last data point 
         * in a curve, 40 characters per curve */
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string Title1;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string Title2;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string Title3;
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string XAxis;   // X Axis label.
        [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 80)]
        public string YAxis;   // Y Axis label.
        public void Initialize()
        {
            NP = new int[100];
            LineType = new int[100];
            SymbolType = new int[100];
        }
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct DPLOT_PLOTMETRICSEX
    {
        public int size;            // Size of this structure, should be filled in by the caller
        public int hll;             // horizontal and
        public int vll;             //   vertical coordinates of the lower left corner of the plot, in pixels
        public int hur;             // horizontal and
        public int vur;             //   vertical coordinates of the upper right corner of the plot, in pixels
        public float xlo;           // value of x at the left plot extent
        public float ylo;           // value of y at the bottom plot extent
        public float xhi;           // value of x at the right plot extent
        public float yhi;           // value of y at the top plot extent
        public float stretch_x;
        public float stretch_y;
        public float stretch_z;
        public float scale;
        public float azimuth;
        public float elevation;
    }

    [StructLayout(LayoutKind.Sequential)]
    public struct RECT
    {
        public int left;
        public int top;
        public int right;
        public int bottom;
    }

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_3DBorder(
        int DocNum, int NumPoints, ref double Edges);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_AddData(
        int DocNum, int DataType, int NumPts, int Curve, ref float x, ref float y);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_AddData8(
        int DocNum, int DataType, int NumPts, int Curve, ref double x, ref double y);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_Command(
        int DocNum, string cmds);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern IntPtr DPlot_GetBitmap(
        int DocNum, int cx, int cy);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern IntPtr DPlot_GetBitmapEx(
        int DocNum, int cx, int cy, ref DPLOT_PLOTMETRICSEX DPM);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern IntPtr DPlot_GetEnhMetaFile(
        int DocNum, float cx, float cy);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_GetVersion(
        StringBuilder Version);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_MinVersion(
        int digit1, int digit2, int digit3, int digit4);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_Plot(
        ref DPLOT d, ref float x, ref float y, string cmds);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_Plot8(
        ref DPLOT d, ref double x, ref double y, string cmds);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_PlotBitmap(
        ref DPLOT d, ref float x, ref float y, string cmds, int cx, int cy);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_PlotBitmap8(
        ref DPLOT d, ref double x, ref double y, string cmds, int cx, int cy);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_PlotToRect(
        ref DPLOT d, ref float x, ref float y, string cmds, IntPtr hwnd, ref RECT rc);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_PlotToRect8(
        ref DPLOT d, ref double x, ref double y, string cmds, IntPtr hwnd, ref RECT rc);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_Request(
        int DocNum, string Item, StringBuilder Value, ref int MaxCount);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_SetErrorMethod(
        int method);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern IntPtr DPlot_Start(
        int Hide, ref int WasActive);

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern void DPlot_Stop();

    [System.Runtime.InteropServices.DllImport("dplotlib.dll")]
    public static extern int DPlot_ZFromXY(
        int DocNum, double x, double y, ref double z);

    [System.Runtime.InteropServices.DllImport("kernel32.dll", CharSet = CharSet.Ansi)]
    public static extern int FreeLibrary(int hLib);

    [System.Runtime.InteropServices.DllImport("kernel32.dll", CharSet = CharSet.Ansi)]
    public static extern int LoadLibraryA(string lpLibFileName);

    // Scaling codes
    public const int SCALE_LINEARX_LINEARY = 1;
    public const int SCALE_LINEARX_LOGY = 2;
    public const int SCALE_LOGX_LINEARY = 3;
    public const int SCALE_LOGX_LOGY = 4;
    public const int SCALE_TRIPARTITE = 5;
    public const int SCALE_LINEARX_PROBABILITY = 6;
    public const int SCALE_GRAINSIZE_DIST = 7;
    public const int SCALE_POLAR = 8;
    public const int SCALE_BARCHART = 9;
    public const int SCALE_LOGX_PROBABILITY = 10;
    public const int SCALE_PROBX_LINEARY = 11;
    public const int SCALE_PROBX_LOGY = 12;
    public const int SCALE_PROBX_PROBY = 13;
    public const int SCALE_TRIANGLE_PLOT = 14;
    public const int SCALE_N185 = 15;

    // Unit-specific scaling codes ... combine with above values using OR operator
    public const int UNITS_DEFAULT = 0;
    public const int UNITS_TRIPARTITE_INCHES = UNITS_DEFAULT;       // Velocity (Y) in inches/sec
    public const int UNITS_TRIPARTITE_FEET = 0x00000100;            //                 feet/sec
    public const int UNITS_TRIPARTITE_MILLIMETERS = 0x00000200;     //                 mm/sec
    public const int UNITS_TRIPARTITE_CENTIMETERS = 0x00000300;     //                 cm/sec
    public const int UNITS_TRIPARTITE_METERS = 0x00000400;          //                 meters/sec
    public const int UNITS_GRAINSIZE_MILLIMETERS = UNITS_DEFAULT;   // Grain sizes (X) in mm
    public const int UNITS_GRAINSIZE_INCHES = 0x00000100;           //                 inches
    public const int UNITS_POLAR_RADIANS = UNITS_DEFAULT;           // Rotation (X) in radians
    public const int UNITS_POLAR_DEGREES = 0x00000100;              //                 degrees
    public const int UNITS_USERDEFINED = 0x00007F00;

    // Data organization:
    public const int DATA_XYXY = 0;             // One or more sets of X,Y data
    public const int DATA_DXY = 1;              // One or more X,Y curves. Constant spacing in X and same number of points in all curves.
    public const int DATA_XYYY = 2;             // One or more X,Y curves. All curves have the same X values.
    public const int DATA_3D = 3;               // Z values on a rectangular grid
    public const int DATA_3DR = 4;              // Random X,Y,Z values
    public const int DATA_IMAGE = 5;            // Used only by DPlot - there's no way to SEND DPlot an image
    public const int DATA_1D = 6;               // One or more groups of Y values.
    public const int DATA_3DS = 7;              // 3D scatter plot

    // Line styles:
    public const int LINESTYLE_NONE = 0;
    public const int LINESTYLE_SOLID = 1;
    public const int LINESTYLE_LONGDASH = 2;
    public const int LINESTYLE_DOTTED = 3;
    public const int LINESTYLE_DASHDOT = 4;
    public const int LINESTYLE_MEDDASH = 5;
    public const int LINESTYLE_DASHDOTDOT = 6;
    public const int LINESTYLE_DASHDOTDOTDOT = 7;

    // Symbol styles
    public const int SYMBOLSTYLE_NONE = 0;
    public const int SYMBOLSTYLE_DOT = 1;
    public const int SYMBOLSTYLE_CROSS = 2;
    public const int SYMBOLSTYLE_ASTERISK = 3;
    public const int SYMBOLSTYLE_X = 4;
    public const int SYMBOLSTYLE_SQUARE = 5;
    public const int SYMBOLSTYLE_DIAMOND = 6;
    public const int SYMBOLSTYLE_TRIANGLE = 7;
    public const int SYMBOLSTYLE_OCTAGON = 8;
    public const int SYMBOLSTYLE_ITRIANGLE = 9;
    public const int SYMBOLSTYLE_HEXAGON = 10;
    public const int SYMBOLSTYLE_PENTAGON = 11;
    public const int SYMBOLSTYLE_STAR = 12;
    public const int SYMBOLSTYLE_FILL = 0x00000100; // May be combined with other styles
    // See DPLOT.HLP for additional symbol styles

    public static void DPlot_FindDPLOTLIB()
    {
        // Find DPLOTLIB.DLL
        int hLib;
        string path;

        hLib = LoadLibraryA("dplotlib.dll");
        if (hLib != 0)
        {
            /* If LoadLibrary finds it, then so will C#. So there's no
             * need to check the registry or set the current directory. */
            FreeLibrary(hLib);
            return;
        }

        Microsoft.Win32.RegistryKey key =
            Microsoft.Win32.Registry.CurrentUser.OpenSubKey(
            "Software\\DPLOT\\DPLOTLIB\\Folder");
        if (key != null)
        {
            path = key.GetValue("").ToString();
            key.Close();
            System.Environment.CurrentDirectory = path;
        }
    }
}
