using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Imaging;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;

namespace test4
{
    public partial class Form1 : Form
    {
        [System.Runtime.InteropServices.DllImport("gdi32.dll")]
        public static extern bool DeleteEnhMetaFile(IntPtr hObject);
        [System.Runtime.InteropServices.DllImport("gdi32.dll")]
        public static extern bool PlayEnhMetaFile(IntPtr hdc, IntPtr hemf, ref Rectangle rc);

        double A;
        double StartX;
        double EndX;
        float SizeX, SizeY;
        int DocNum = 0;
        int NP;
        int NP3DR;
        int NX3D;
        int NY3D;
        int dxPic, dyPic;
        int WasDPlotActive = 0;
        IntPtr hemf;

        public Form1()
        {
            InitializeComponent();
        }

        private void PaintMyMetafile()
        {
            /* While we COULD simply convert the DPlot-produced HENHMETAFILE
             * to a MetaFile object with:
             * 
             * Metafile mf = new Metafile(hemf,true);
             * 
             * and draw it using
             * pictureBox1.Image = mf;
             * 
             * doing so draws the metafile full-scale, w/o stretching or
             * shrinking. We try to get the actual physical size of the
             * picturebox below, but the DpiX and DpiY values don't seem
             * to be exact. On one system this was tried on, they're off
             * by about 25%. The result is a metafile that doesn't fill
             * the picturebox.
             * 
             * So... we're instead using Win32's PlayEnhMetaFile in our
             * pictureBox1_Paint routine (see below).
             */
            if (hemf != null)
            {
                DeleteEnhMetaFile(hemf);
                hemf = (IntPtr)null;
            }
            // Size of the metafile to retrieve, in inches.
            Graphics picGraphics = pictureBox1.CreateGraphics();
            SizeX = (float)pictureBox1.Size.Width / (float)picGraphics.DpiX;
            SizeY = (float)pictureBox1.Size.Height / (float)picGraphics.DpiY;

            hemf = dplot.DPlot_GetEnhMetaFile(DocNum, SizeX, SizeY);
            dplot.DPlot_Command(DocNum, "[FileClose()]");
            pictureBox1.Invalidate();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.FormClosing += new FormClosingEventHandler(Form1_FormClosing);
            this.Resize += new EventHandler(Form1_Resize);
            pictureBox1.Paint += new PaintEventHandler(pictureBox1_Paint);

            // XY plot variables
            A = 0.5;
            StartX = 0.0;
            EndX = 4.0;
            NP = 1001;
            textBox1.Text = String.Format("{0}", A);
            textBox2.Text = String.Format("{0}", StartX);
            textBox3.Text = String.Format("{0}", EndX);
            textBox4.Text = String.Format("{0}", NP);
            // 3D random points
            NP3DR = 10000;
            textBox5.Text = String.Format("{0}", NP3DR);
            // 3D gridded data
            NX3D = NY3D = 50;
            textBox6.Text = String.Format("{0}", NX3D);
            textBox7.Text = String.Format("{0}", NY3D);

            labelTime.Text = "";

            dplot.DPlot_FindDPLOTLIB();
            dplot.DPlot_SetErrorMethod(2);
            dplot.DPlot_Start(1, ref WasDPlotActive);

            dxPic = this.Size.Width - pictureBox1.Size.Width;
            dyPic = this.Size.Height - pictureBox1.Size.Height;
        }
        private void Form1_Resize(object sender, System.EventArgs e)
        {
            if (this.Size.Width > dxPic && this.Size.Height > dyPic)
                pictureBox1.Size = new Size(this.Size.Width - dxPic, this.Size.Height - dyPic);
        }
        void Form1_FormClosing(object Sender, FormClosingEventArgs e)
        {
            if (hemf != null) DeleteEnhMetaFile(hemf);
            if (WasDPlotActive == 0) dplot.DPlot_Stop();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            double dX, Xi, xTick;
            double[] X;
            double[] Y;
            int i;
            string cmds;

            Stopwatch timeoperation = Stopwatch.StartNew();
            timeoperation.Start();

            A = double.Parse(textBox1.Text);
            StartX = double.Parse(textBox2.Text);
            EndX = double.Parse(textBox3.Text);
            NP = int.Parse(textBox4.Text);

            if (NP < 2)
            {
                MessageBox.Show("The number of points must be greater than 1.", "Demo");
                return;
            }
            X = new double[2];
            Y = new double[NP * 2];

            dX = (EndX - StartX) / (NP - 1);
            X[0] = dX;
            X[1] = StartX;
            for (i = 0; i < NP; i++)
            {
                Xi = StartX + i * dX;
                Y[i] = Math.Sin(A * Math.PI * Xi);
                Y[i + NP] = Math.Cos(A * Math.PI * Xi);
            }
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_DXY;
            dp.MaxCurves = 2;           // Must be >= number of curves we plot 
            dp.MaxPoints = NP;          // Anything >= NP will do 
            dp.NumCurves = 2;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.LegendX = 0.05F;
            dp.LegendY = 0.05F;
            dp.NP[0] = NP;
            dp.NP[1] = NP;
            dp.LineType[0] = dplot.LINESTYLE_SOLID;
            dp.LineType[1] = dplot.LINESTYLE_LONGDASH;
            dp.Title1 = "Data sent to DPlot via DPLOTLIB.DLL";
            dp.XAxis = "x";
            dp.YAxis = "y";
            // Take a stab at a reasonable tick mark interval
            xTick = 1.0;
            while ((EndX - StartX) / xTick < 5)
                xTick /= 2;
            while ((EndX - StartX) / xTick > 12)
                xTick *= 2;
            /* The command string can be as complex as you like, limited to 32K characters.
            As an alternative, if your program will be producing many similar plots then
            you might prefer to create a preferences file, edit the file with a text 
            editor to remove unwanted entries, and use the [GetPreferences("filename")]
            command. 
            */
            cmds = "[ManualScale(" +
                String.Format("{0}", StartX) + ",-1.25," +
                String.Format("{0}", EndX) + ",1.25)]" +
                "[TickInterval(1," + String.Format("{0}", xTick) + ",0.25)]" +
                "[Caption(\"DPLOTLIB XY Plot\")]" +
                "[Legend(1,\"sin(" + (A == 1.0 ? "" : String.Format("{0}", A)) + "{\\sp}x)\")]" +
                "[Legend(2,\"cos(" + (A == 1.0 ? "" : String.Format("{0}", A)) + "{\\sp}x)\")]" +
                "[Color(1,255,0,0)][Color(2,0,0,255)]" +
                "[TextFont(1, 8,  0,0,0,0,0,\"Arial\")]" +
                "[TextFont(2,12,700,0,0,0,0,\"Arial\")]" +
                "[TextFont(4,10,  0,0,0,0,0,\"Arial\")]" +
                "[TextFont(5,10,  0,0,0,0,0,\"Arial\")]" +
                "[TextFont(6,10,  0,0,0,0,0,\"Arial\")]" +
                "[DocMaximize()][ClearEditFlag()]";
            DocNum = dplot.DPlot_Plot8(ref dp, ref X[0], ref Y[0], cmds);
            if (DocNum != 0) PaintMyMetafile();
            timeoperation.Stop();
            labelTime.Text = String.Format("Elapsed: {0} msec", timeoperation.ElapsedMilliseconds);
        }

        private void button2_Click(object sender, EventArgs e)
        {
            double[,] Node;
            double dummy = 0;
            int i;
            string cmds;
            Random rand;

            Stopwatch timeoperation = Stopwatch.StartNew();
            timeoperation.Start();

            NP3DR = int.Parse(textBox5.Text);
            if (NP3DR < 3)
            {
                MessageBox.Show("The number of points must be 3 or greater.", "Demo");
                return;
            }

            Node = new double[NP3DR, 3];
            rand = new Random(1);
            for (i = 0; i < NP3DR; i++)
            {
                Node[i, 0] = -3 + 6 * rand.NextDouble();
                Node[i, 1] = -3 + 6 * rand.NextDouble();
                Node[i, 2] = Math.Sin(Node[i, 0]) * Math.Cos(Node[i, 1]);
            }
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3DR;
            dp.MaxCurves = 1;
            dp.MaxPoints = NP3DR;
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via DPLOTLIB.DLL";
            dp.Title2 = "z = sin(x)*cos(y)";
            dp.XAxis = "x";
            dp.YAxis = "y";
            cmds = "[Caption(\"DPLOTLIB Surface Plot\")]" +
                "[Contour3D(0)][ContourGrid(0)][ContourAxes(1)]" +
                "[ContourMethod(1)][ContourLevels(21,-1,1)]" +
                "[ContourScales(1,1,1)][ManualScale(-3,-3,3,3,-1,1)]" +
                "[Size(0,0,0,0)]"+
                "[FontPoints(1,8)][FontPoints(2,12)][FontPoints(3,12)]" +
                "[FontPoints(4,10)][FontPoints(5,10)][FontPoints(6,8)][FontPoints(8,10)]" +
                "[DocMaximize()]";

            DocNum = dplot.DPlot_Plot8(ref dp, ref dummy, ref Node[0, 0], cmds);
            if (DocNum != 0) PaintMyMetafile();
            timeoperation.Stop();
            labelTime.Text = String.Format("Elapsed: {0} msec", timeoperation.ElapsedMilliseconds);
        }

        private void button3_Click(object sender, EventArgs e)
        {
            double[,] Z;
            double[] extents;
            double dX, dY, X, Y;
            int i, j;
            string cmds;

            Stopwatch timeoperation = Stopwatch.StartNew();
            timeoperation.Start();

            NX3D = int.Parse(textBox6.Text);
            NY3D = int.Parse(textBox7.Text);
            if (NX3D < 1)
            {
                MessageBox.Show("NX must be greater than 0.", "Demo");
                return;
            }
            if (NY3D < 1)
            {
                MessageBox.Show("NY must be greater than 0.", "Demo");
                return;
            }

            Z = new double[NX3D + 1, NY3D + 1];
            extents = new double[4];
            extents[0] = -3;        // xlo
            extents[1] = -3;        // ylo
            extents[2] = 3;         // xhi
            extents[3] = 3;         // yhi
            dX = (extents[2] - extents[0]) / NX3D;
            dY = (extents[3] - extents[1]) / NY3D;
            for (i = 0; i < NX3D + 1; i++)
            {
                X = extents[0] + i * dX;
                for (j = 0; j < NY3D + 1; j++)
                {
                    Y = extents[1] + j * dY;
                    Z[i, j] = Math.Sin(X) * Math.Cos(Y);
                }
            }
            dplot.DPLOT dp = new dplot.DPLOT();
            dp.Initialize();
            dp.Version = dplot.DPLOT_DDE_VERSION;
            dp.DataFormat = dplot.DATA_3D;
            dp.MaxCurves = NX3D;
            dp.MaxPoints = NY3D;
            dp.NumCurves = 1;
            dp.ScaleCode = dplot.SCALE_LINEARX_LINEARY;
            dp.Title1 = "Data sent to DPlot via DPLOTLIB.DLL";
            dp.Title2 = "z = sin(x)*cos(y)";
            dp.XAxis = "x";
            dp.YAxis = "y";
            cmds = "[Caption(\"DPLOTLIB Surface Plot\")]" +
                "[Contour3D(0)][ContourGrid(0)][ContourAxes(1)]" +
                "[ContourMethod(1)][ContourLevels(21,-1,1)]" +
                "[ContourScales(1,1,1)][ManualScale(-3,-3,3,3,-1,1)]" +
                "[Size(0,0,0,0)]" +
                "[FontPoints(1,8)][FontPoints(2,12)][FontPoints(3,12)]" +
                "[FontPoints(4,10)][FontPoints(5,10)][FontPoints(6,8)][FontPoints(8,10)]" +
                "[DocMaximize()]";
            DocNum = dplot.DPlot_Plot8(ref dp, ref extents[0], ref Z[0, 0], cmds);
            if (DocNum != 0) PaintMyMetafile();
            timeoperation.Stop();
            labelTime.Text = String.Format("Elapsed: {0} msec", timeoperation.ElapsedMilliseconds);
        }
        private void pictureBox1_Paint(object sender, System.Windows.Forms.PaintEventArgs e)
        {
            /* In this demo if you resize the form, the picturebox is 
             * also resized, causing this routine to be called again. This
             * IS what we want, but in this demo we don't do anything to
             * scale the output rectangle (rcPic below) such that it is the
             * same aspect ratio as the metafile. So you'll sometimes get a 
             * stretched picture that looks... sometimes not so good. If 
             * this is important to you, then correct rcPic to keep the
             * same width/height ratio as SizeX/SizeY. */
            IntPtr hdc;
            Graphics g = e.Graphics;
            if (hemf != null)
            {
                pictureBox1.Image = null;
                Rectangle rcPic = pictureBox1.ClientRectangle;
                g.Clear(Color.White);
                hdc = g.GetHdc();
                PlayEnhMetaFile(hdc, hemf, ref rcPic);
                g.ReleaseHdc(hdc);
            }
        }
    }
}