'*********************************************************************
' This is File fb-dplotlib.bi - include file for DplotJr
'
' Date: 25 Nov 2005
'
' Purpose:
'  To provide include file for DplotJr DLL
'
' Author(s)
'  Translated from supplied source by
'  T. McCloskey, Clifton Park, NY 12065
'
' Note:
'  1) At the time of this writing (24 Nov 05), the latest stable (official)
'     release of FreeBASIC is version 0.14. There is a testing revision
'     available for the next version (0.15) and this (and all testing versions)
'     should be considered unstable prior to official release. However, it is
'     mentioned here as this planned revision/release makes some significant
'     changes to the windows include files. Take note and adjust for the proper
'     include statements below.
'
'  2) Please note the comments about fixed length strings in FreeBASIC's
'     TYPE structures. FreeBASIC stores strings with trailing nulls much
'     like "C" . This storage scheme also applies within TYPE fields and
'     the strings will be store within the structure including the trailing
'     null. To confuse matters further, the field length specification only
'     specifies the FULL DATA LENGTH of the field element. Hence, a string
'     length specified in the field is the length of data ONLY but will
'     actually be padded with an addition (null) byte. Hence, a string field
'     specified as "STRING * 7" will actually be constructed with a field
'     of 8 bytes (7 bytes of data + 1 byte \null).

'---------------------------------------------------------------------

'Use the appropiate FreeBASIC include files for general Windows constants
'(will automatically adjust for versions prior to 0.15)

#IF (__FB_VERSION__ > "0.14" )
#INCLUDE ONCE "C:\freebasic\inc\windows.bi"
#ELSE
#INCLUDE ONCE "c:\freebasic\inc\win\gdi32.bi"
#INCLUDE ONCE "c:\freebasic\inc\win\user32.bi"
#INCLUDE ONCE "c:\freebasic\inc\win\winbase.bi"
#INCLUDE ONCE "c:\freebasic\inc\win\commdlg32.bi"
#INCLUDE ONCE "c:\freebasic\inc\win\kernel32.bi"
#INCLUDE ONCE "c:\freebasic\inc\win\winver.bi"
#ENDIF
'
' SetBkMode constants
#Define TRANSPARENT     1
#Define OPAQUE          2
#Define NEWTRANSPARENT  3

'
' DPlot-specific stuff
'
#Define DPLOT_DDE_VERSION  3    ' Version 3 structure allows up to 100 data sets per plot

Type DPLOT
    version    As Long          ' version number of this structure, should be set by caller to DPLOT_DDE_VERSION
    hwnd       As Long          ' handle of client application window
    DataFormat As Long          ' XY pairs, DX and Y, etc. See DATA_ constants
    MaxCurves  As Long           ' DataFormat    Description
                                ' DATA_XYXY     Max. number of curves, <= 100
                                ' DATA_DXY      ..
                                ' DATA_XYYY     ..
                                ' DATA_3D       Number of grid cells in X direction,
                                '                 = number of data columns-1
                                ' DATA_3DR      not used
                                ' DATA_1D       Max. number of groups, <= 100
                                ' DATA_3DS      Max. number of data sets, <= 100
    MaxPoints  As Long          ' DATA_XYXY     Max. number of points/curve
                                ' DATA_DXY      ..
                                ' DATA_XYYY     ..
                                ' DATA_3D       Number of grid cells in Y direction,
                                '                 = number of data rows-1
                                ' DATA_3DR      Number of X,Y,Z triplets
                                ' DATA_1D       Max. number of points/group
                                ' DATA_3DS      Max. number of points/data set
    NumCurves  As Long          ' Actual number of curves/groups, always 1 for
                                '   DATA_3D or DATA_3DR
    ScaleCode  As Long          ' scaling code for XY plots (Linear, Log, etc.)
    LegendX    As Single        ' left coord of legend, expressed as a ratio
                                '    of plot size (0->1) (CAN be negative and/or > 1,
                                '    but exact placement is then a bit hard to predict)
    LegendY    As Single        ' top coord of legend, again expressed as a ratio
                                '    of plot size
    NP(0 To 99) As Long         ' Actual number of points in each curve/group. Not used for DATA_3D.
                                '   Cannot exceed MaxPoints.
    LineType(0 To 99)   As Long ' Line types (see codes below)
    SymbolType(0 To 99) As Long ' Symbol types (see codes below)
    SizeofExtraInfo As Long     ' Extra information following X,Y data. No need to
                                '   fill in this member for DPLOTLIB calls

    ' ************** IMPORTANT FreeBASIC Note **********************************
    ' ****  The following string field lengths have been decreased by one. *****
    ' ****  This is to handle a quirk in FreeBASIC and the way it handles  *****
    ' ****  fixed length strings in TYPE fields. The actual data that will *****
    ' ****  be passed will be one more than specified (adds trailing NULL).*****
    ' ****  Hence, the intended field lengths passed will be correct (!)   *****
    ' **************************************************************************

    Legend (0 To 100) As String * 79    ' Legend(0) is the caption for the legend
                                        '   Legend(n) is the caption for the n'th curve
    Label(0 To 99)    As String * 39    ' Strings displayed beside the last data point
                                        '   in a curve, 40 characters per curve
    Title1 As String * 79       ' Chart first Title
    Title2 As String * 79       ' Chart second Title
    Title3 As String * 79       ' Chart third Title
    XAxis  As String * 79       ' X Axis label.
    YAxis  As String * 79       ' Y Axis label.
End Type

Type DPLOT_PLOTMETRICS
    SIZE As Long        ' size of this structure, should be filled in by the caller
    hll As Long         ' horizontal and
    vll As Long         '  vertical coordinates of the lower left corner of the plot, in pixels
    hur As Long         ' horizontal and
    vur As Long         '  vertical coordinates of the upper right corner of the plot, in pixels
    xlo As Single       ' value of x at the left plot extent
    ylo As Single       ' value of y at the bottom plot extent
    xhi As Single       ' value of x at the right plot extent
    yhi As Single       ' value of y at the top plot extent
End Type

Type DPLOT_PLOTMETRICSEX
    SIZE As Long        ' size of this structure, should be filled in by the caller
    hll As Long         ' horizontal and
    vll As Long         '  vertical coordinates of the lower left corner of the plot, in pixels
    hur As Long         ' horizontal and
    vur As Long         '  vertical coordinates of the upper right corner of the plot, in pixels
    xlo As Single       ' value of x at the left plot extent
    ylo As Single       ' value of y at the bottom plot extent
    xhi As Single       ' value of x at the right plot extent
    yhi As Single       ' value of y at the top plot extent
    stretch(0 To 2) As Single
    scale As Single
    azimuth As Single
    elevation As Single
End Type

Declare Function DPlot_3DBorder Lib "dplotlib" alias "DPlot_3DBorder" (ByVal DocNum As Long, ByVal NumPts As Long, ByRef points As Double) As Long
Declare Function DPlot_AddData Lib "dplotlib" alias "DPlot_AddData" (ByVal DocNum As Long, ByVal DataType As Long, ByVal NumPts As Long, ByVal Curve As Long, ByRef X As Single, ByRef Y As Single) As Long
Declare Function DPlot_AddData8 Lib "dplotlib" alias "DPlot_AddData8" (ByVal DocNum As Long, ByVal DataType As Long, ByVal NumPts As Long, ByVal Curve As Long, ByRef X As Double, ByRef Y As Double) As Long
Declare Function DPlot_Command Lib "dplotlib" alias "DPlot_Command" (ByVal DocNum As Long, ByVal Command As String) As Long
Declare Function DPlot_GetBitmap Lib "dplotlib" alias "DPlot_GetBitmap" (ByVal DocNum As Long, ByVal cx As Long, ByVal cy As Long) As Long
Declare Function DPlot_GetBitmapEx Lib "dplotlib" alias "DPlot_GetBitmapEx" (ByVal DocNum As Long, ByVal cx As Long, ByVal cy As Long, ByRef DPM As Any) As Long
Declare Function DPlot_GetEnhMetaFile Lib "dplotlib" alias "DPlot_GetEnhMetaFile" (ByVal DocNum As Long, ByVal cx As Single, ByVal cy As Single) As Long
Declare Function DPlot_GetVersion Lib "dplotlib" alias "DPlot_GetVersion" (ByVal version As String) As Long
Declare Function DPlot_MinVersion Lib "dplotlib" alias "DPlot_MinVersion" (ByVal digit1 As Long, ByVal digit2 As Long, ByVal digit3 As Long, ByVal digit4 As Long) As Long
Declare Function DPlot_Plot Lib "dplotlib" alias "DPlot_Plot" (ByRef d As DPLOT, ByRef X As Single, ByRef Y As Single, ByVal cmds As String) As Long
Declare Function DPlot_Plot8 Lib "dplotlib" alias "DPlot_Plot8" (ByRef d As DPLOT, ByRef X As Double, ByRef Y As Double, ByVal cmds As String) As Long
Declare Function DPlot_Plot8p Lib "dplotlib" alias "DPlot_Plot8p" (ByRef d As DPLOT, ByVal px As Long, ByVal py As Long, ByVal cmds As String) As Long
Declare Function DPlot_PlotBitmap Lib "dplotlib" alias "DPlot_PlotBitmap" (ByRef d As DPLOT, ByRef X As Single, ByRef Y As Single, ByVal cmds As String, ByVal cx As Long, ByVal cy As Long) As Long
Declare Function DPlot_PlotBitmap8 Lib "dplotlib" alias "DPlot_PlotBitmap8" (ByRef d As DPLOT, ByRef X As Double, ByRef Y As Double, ByVal cmds As String, ByVal cx As Long, ByVal cy As Long) As Long
Declare Function DPlot_PlotToRect Lib "dplotlib" alias "DPlot_PlotToRect" (ByRef d As DPLOT, ByRef X As Single, ByRef Y As Single, ByVal cmds As String, ByVal hwnd As Long, ByRef rc As RECT) As Long
Declare Function DPlot_PlotToRect8 Lib "dplotlib" alias "DPlot_PlotToRect8" (ByRef d As DPLOT, ByRef X As Double, ByRef Y As Double, ByVal cmds As String, ByVal hwnd As Long, ByRef rc As RECT) As Long
Declare Function DPlot_Request Lib "dplotlib" alias "DPlot_Request" (ByVal DocNum As Long, ByVal Item As String, ByVal Value As String, ByRef MaxCount As Long) As Long
Declare Sub DPlot_SetErrorMethod Lib "dplotlib" alias "DPlot_SetErrorMethod" (ByVal method As Long)
Declare Function DPlot_Start Lib "dplotlib" alias "DPlot_Start" (ByVal Minimize As Long, ByRef WasActive As Long) As Long
Declare Sub DPlot_Stop Lib "dplotlib" alias "DPlot_Stop" ()
Declare Function DPlot_ZFromXY Lib "dplotlib" alias "DPlot_ZFromXY" (ByVal DocNum As Long, ByVal X As Double, ByVal Y As Double, ByRef Z As Double) As Long

' XY Scaling codes:

#Define SCALE_LINEARX_LINEARY      1
#Define SCALE_LINEARX_LOGY         2
#Define SCALE_LOGX_LINEARY         3
#Define SCALE_LOGX_LOGY            4
#Define SCALE_TRIPARTITE           5
#Define SCALE_LINEARX_PROBABILITY  6
#Define SCALE_GRAINSIZE_DIST       7
#Define SCALE_POLAR                8
#Define SCALE_BARCHART             9
#Define SCALE_LOGX_PROBABILITY    10
#Define SCALE_PROBX_LINEARY       11
#Define SCALE_PROBX_LOGY          12
#Define SCALE_PROBX_PROBY         13
#Define SCALE_TRIANGLE_PLOT       14
#Define SCALE_N185                15

' Unit-specific scaling codes ... combine with above values using OR operator

#Define UNITS_DEFAULT                 &H000
#Define UNITS_TRIPARTITE_INCHES       UNITS_DEFAULT      ' Velocity (Y) in inches/sec
#Define UNITS_TRIPARTITE_FEET         &H100              '                 feet/sec
#Define UNITS_TRIPARTITE_MILLIMETERS  &H200              '                 mm/sec
#Define UNITS_TRIPARTITE_CENTIMETERS  &H300              '                 cm/sec
#Define UNITS_TRIPARTITE_METERS       &H400              '                 meters/sec
#Define UNITS_GRAINSIZE_MILLIMETERS   UNITS_DEFAULT      ' Grain sizes (X) in mm
#Define UNITS_GRAINSIZE_INCHES        &H100              '                 inches
#Define UNITS_POLAR_RADIANS           UNITS_DEFAULT      ' Rotation (X) in radians
#Define UNITS_POLAR_DEGREES           &H100              '                 degrees
#Define UNITS_USERDEFINED             &H7F00

' Data organization following DPLOT structure:

#Define DATA_XYXY   0          ' One or more sets of X,Y data
#Define DATA_DXY    1          ' One or more X,Y curves. Constant spacing in X and same number of points in all curves.
#Define DATA_XYYY   2          ' One or more X,Y curves. All curves have the same X values.
#Define DATA_3D     3          ' Z values on a rectangular grid
#Define DATA_3DR    4          ' Random X,Y,Z values
#Define DATA_IMAGE  5          ' Used only by DPlot - there's no way to SEND DPlot an image
#Define DATA_1D     6          ' One or more groups of Y values.
#Define DATA_3DS    7          ' 3D scatter plot

' Line styles:
#Define LINESTYLE_NONE           0
#Define LINESTYLE_SOLID          1
#Define LINESTYLE_LONGDASH       2
#Define LINESTYLE_DOTTED         3
#Define LINESTYLE_DASHDOT        4
#Define LINESTYLE_MEDDASH        5
#Define LINESTYLE_DASHDOTDOT     6
#Define LINESTYLE_DASHDOTDOTDOT  7

' Symbol styles
#Define SYMBOLSTYLE_NONE         0
#Define SYMBOLSTYLE_DOT          1
#Define SYMBOLSTYLE_CROSS        2
#Define SYMBOLSTYLE_ASTERISK     3
#Define SYMBOLSTYLE_X            4
#Define SYMBOLSTYLE_SQUARE       5
#Define SYMBOLSTYLE_DIAMOND      6
#Define SYMBOLSTYLE_TRIANGLE     7
#Define SYMBOLSTYLE_OCTAGON      8
#Define SYMBOLSTYLE_ITRIANGLE    9
#Define SYMBOLSTYLE_HEXAGON     10
#Define SYMBOLSTYLE_PENTAGON    11
#Define SYMBOLSTYLE_STAR        12
#Define SYMBOLSTYLE_FILL     &H100       ' May be combined with other styles

' For information only - the following is the contents for the
' Dynamic Link Library Definition File (DPLOTLIB.DLL.DEF) which allows
' proper reference from FreeBASIC decorated names to DPLOTLIB.DLL
' internal names
  '---------- DPLOTLIB.DLL.DEF ------------------------------------------
  'LIBRARY dplotlib.dll
  'EXPORTS
  'DPlot_3DBorder@12 = DPlot_3DBorder
  'DPlot_AddData@24 = DPlot_AddData
  'DPlot_AddData8@24 = DPlot_AddData8
  'DPlot_Command@8 = DPlot_Command
  'DPlot_GetBitmap@12 = DPlot_GetBitmap
  'DPlot_GetBitmapEx@16 = DPlot_GetBitmapEx
  'DPlot_GetEnhMetaFile@12 = DPlot_GetEnhMetaFile
  'DPlot_GetVersion@4 = DPlot_GetVersion
  'DPlot_MinVersion@16 = DPlot_MinVersion
  'DPlot_Plot@16 = DPlot_Plot
  'DPlot_Plot8@16 = DPlot_Plot8
  'DPlot_Plot8p@16 = DPlot_Plot8p
  'DPlot_PlotBitmap@24 = DPlot_PlotBitmap
  'DPlot_PlotBitmap8@24 = DPlot_PlotBitmap8
  'DPlot_PlotToRect@24 = DPlot_PlotToRect
  'DPlot_PlotToRect8@24 = DPlot_PlotToRect8
  'DPlot_Request@16 = DPlot_Request
  'DPlot_SetErrorMethod@4 = DPlot_SetErrorMethod
  'DPlot_Start@8 = DPlot_Start
  'DPlot_Stop@0 = DPlot_Stop
  'DPlot_ZFromXY@16 = DPlot_ZFromXY
  ' --------- END OF FILE -------------------------------------------

' To Construct the import interface file "libDPLOTLIB.DLL.a",
' utilize "DLLTOOL" (usually located in the "FreeBASIC\BIN\WIN32"
' directory) and the following command to construct the import
' library using above DEF file:
'
'   c:\freebasic\bin\win32\dlltool -d dplotlib.dll.def -D dplotlib.dll -l libdplotlib.dll.a
