c vim: sw=2 ts=8 co=132 nowrap tw=0   
c
c Salford Software FTN95 notes
c
c With the exception of the standard caveats of the switches used to call c
c programs, this program follows the fortran 77 standard.  Use this program
c if the compiler does not understand the VAX structure statements and you 
c cant use fortran 90/95.
c
c One place the program doesnt follow the standard is in the include file, 
c dplot.h.  the DPLOT structure is created using a common block.  The 
c standard doesnt allow mixing of character, reals, and integers in a 
c common block. I dont know of any compilers that dont support this extension,
c so it shouldnt cause any problems.
C
C Compile and link with: makefile.bat
C
C Explicitly dimension arrays
C
C Change structures to common blocks
C
C Change NULL_POINTER to a real*4 declaration
C
C Interface to DPLOTLIB functions is included in dplot.h
C
C end  Salford Software notes
c
c FTEST - tests DPLOTLIB.DLL in Fortran.
c
      program ftest
      implicit none
      include  'dplot.fi'
c
      integer option

 100  continue

      write(*,*) 'Enter 0 to exit'
      write(*,*) '      1 for XY plot example'
      write(*,*) '      2 for contour plot of gridded data'
      write(*,*) '      3 for contour plot of randomly-spaced points'
      write(*,*) '      4 for box-and-whisker example'
      write(*,*) '      5 for a contour plot with concave border'
      write(*,*) '      6 for a 3D scatter plot'
      write(*,*) '? '
      read(*,*) option

      select case(option)
      case(1)
        call XYExample()

      case(2)
        call ContourPlot1()

      case(3)
        call ContourPlot2()

      case(4)
        call BoxAndWhiskerPlot()

      case(5)
        call ConcaveBorderExample()

      case(6)
        call ThreeDScatter()

      case(0)
        stop
  
      end select

      goto 100
      end
c
c=======================================================================
c
      real*8 function random()
      implicit none
      integer  seed /75347/

      seed = mod(seed*7141+54773,259200)
      random = dble(seed)/259200.
      return
      end
c
c=======================================================================
c
      subroutine XYExample()
      implicit none
      include  'dplot.fi'
c
c     Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
c     In this case we've used DATA_XYYY (one X array for one or more Y arrays).
c
c     Since the X values are evenly spaced we could also use
c     DATA_DXY  - X has only 2 elements, DX and X0
c
c     And you can ALWAYS use
c     DATA_XYXY - Each curve uses its own X array. This is the only
c                 option available if the curves have different X values or
c                 a different number of points.

      integer*4 NP
      parameter (NP=1001)
      integer*4 i
      integer*4 ret
      real*8    x(NP), y(2*NP)
      real*8    PI
      character*32000 cString

      PI = 4.*atan(1.0)
      do i=1,NP
        x(i)   = (4.*i)/(NP-1)
        y(i)   = sin(PI*x(i))
        y(i+NP)= cos(PI*x(i))
      end do

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_XYYY
      DPMaxCurves   = 2             ! Must be >= number of curves we plot
      DPMaxPoints   = NP            ! Anything >= NP will do
      DPNumCurves   = 2
      DPScale       = SCALE_LINEARX_LINEARY
      DPLegendX     = 0.05
      DPLegendY     = 0.05
      DPNP(1)       = NP
      DPNP(2)       = NP
      DPLineType(1) = LINESTYLE_SOLID
      DPLineType(2) = LINESTYLE_LONGDASH
      DPLegend(1)   = 'sin({\sp}x)'     ! {\s is DPlot-speak for "use symbol font"
      DPLegend(2)   = 'cos({\sp}x)'  
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPTitle(2)    = ' '
      DPTitle(3)    = ' '
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
c               
c     The command string can be as complex as you like, limited to 32K characters.
c     As an alternative, if your program will be producing many similar plots then
c     you might prefer to create a preferences file, edit the file with a text 
c     editor to remove unwanted entries, and use the [GetPreferences("filename")]
c     command. 
c
      cString = '[ManualScale(0,-1.25,4,1.25)]'//
     2          '[TickInterval(1,0.5,0.25)]'//
     3          '[Caption("DPLOTLIB XY Test")]'//
     4          '[DocMaximize()]'//
     5          '[ClearEditFlag()]'//char(0) 
      ret = DPlot_Plot8( DPVersion, x, y, cString )
      return
      end
c
c======================================================================
c
      subroutine ContourPlot1()
      implicit none
      include  'dplot.fi'
c
c     Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
c
      integer*4 NX, NY
      parameter (NX=61, NY=41)
      integer*4 i, j, ret
      real*8    extents(4)
      real*8    xx, yy
      real*8    z(NX*NY)

c     extents are, in order, xlo, ylo, xhi, yhi
      extents(1) = -3.
      extents(2) = -2.
      extents(3) =  3.
      extents(4) =  2.
      do i=1,Nx
        xx = extents(1) + ((extents(3)-extents(1))*i)/(Nx-1)
        do j=1,Ny
          yy = extents(2) + ((extents(4)-extents(2))*j)/(Ny-1)
          z((i-1)*Ny+j) = sin(xx)*cos(yy)
        end do
      end do

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3D
c
c     MaxCurves and MaxPoints members are used for the size of the grid, in grid cells
c
      DPMaxCurves   = Nx-1
      DPMaxPoints   = Ny-1
      DPNumCurves   = 1
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPTitle(2)    = 'z = sin(x)*cos(y)'
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
		
      ret = DPlot_Plot8(DPVersion,extents,z,
     1       '[Caption("DPLOTLIB Contour Test 1")]'//
     2       '[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]'//
     3       '[ContourView(30,20)][ContourLevels(20,-1,1)]'//
     4       '[ContourScales(1,1,1)]'//
     5       '[ContourLighting(2,0.2)]'//                     ! Phong shading, 20% ambient light
     6       '[FontPoints(1,10)][FontPoints(6,10)]'//
     7       '[FontPoints(4,12)][FontPoints(5,12)][FontPoints(8,12)]'//
     8       '[ZAxisLabel("z")][DocMaximize()]'//char(0))
      return
	  end
c
c======================================================================
c
      subroutine ContourPlot2()
      implicit none
      include  'dplot.fi'
c
c     Contour plot with randomly spaced points
c
      integer*4 i, ret
      real*8    PI
      real*8    dummy

      integer*4 NZ
      parameter (NZ=1000)
      real*8   Node(3,NZ)

      real*8   random
      external random

      PI = 4.*atan(1.0)
      do i=1,Nz
         Node(1,i) = -PI + 2*PI*random()
         Node(2,i) = -PI + 2*PI*random()
         Node(3,i) = sin(Node(1,i))*cos(Node(2,i))
      end do

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3DR
      DPMaxCurves   = 1
      DPMaxPoints   = Nz
      DPNumCurves   = 1
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPTitle(2)    = 'z = sin(x)*cos(y)'
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
		
      ret = DPlot_Plot8(DPVersion,dummy,Node,
     1           '[Caption("DPLOTLIB Contour Test 2")]'//
     2           '[Contour3D(0)][ContourGrid(0)]'//
     3           '[ContourLevels(21,-1,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine BoxAndWhiskerPlot()
      implicit none
      include  'dplot.fi'
c
c     Box-and-whisker plot - 3 groups
c
      integer*4 i, j, k, ret
      real*8    dummy

      integer*4 NP
      parameter (NP=150)
      real*8  y(NP)

      real*8   random
      external random

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_1D
      DPMaxCurves   = 3
      DPMaxPoints   = 50
      DPNumCurves   = 3
      DPLegendX		= 0.02
      DPLegendY		= 0.7
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Data sent to DPlot via DPLOTLIB.DLL'
      DPXAxis       = ' '
      DPYAxis       = 'Amplitude'

      do i=1,DPNumCurves
        DPNP(i) = DPMaxPoints
        DPLegend(i) = 'Group '//char(64+i)//char(0)
      end do

c     Generate group data, centered around 5.0 with progressively larger variations
      k = 1
      do i=1,DPNumCurves
        do j=1,DPNP(i)
          y(k) = 5.0 + (i+1)*(-0.5+random())
          k = k+1
        end do
      end do
		
      ret = DPlot_Plot8(DPVersion,dummy,y,
     1           '[Caption("DPLOTLIB Box-and-whisker Test")]'//
     2           '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]'//
     3           '[Stat_BWOptions(1,1,0,1,1.5,2,3)]'//
     4           '[Stat_GrandMean(3,30)]'//
     5           '[FontPoints(1,10)][FontPoints(6,10)]'//
     6           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ConcaveBorderExample()
      implicit none
      include  'dplot.fi'
c
c     Surface plot with concave border
c
      integer*4 i, DocNum, ret
      real*8    x0, y0, side
      real*8    dummy
      real*8    PI
      real*8    Node(3,10)
      real*8    Border(2,10)

      PI = 4.*atan(1.0)

      x0 = 0.             ! center of star/pentagon
      y0 = 0.
      side = 1.0          ! length of each pentagon side

      Node(1,1) = x0 - 0.5*side
      Node(2,1) = y0 - 0.5*side*tan(54*PI/180)
      Node(3,1) = 0.0
      Node(1,2) = x0
      Node(2,2) = Node(2,1)+0.5*side*tan(36*PI/180)
      Node(3,2) = 1.0
      Node(1,3) = x0 + 0.5*side
      Node(2,3) = Node(2,1)
      Node(3,3) = 0.0
      Node(1,4) = Node(1,3)-0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(2,4) = Node(2,3)+0.5*side*(cos(18*PI/180)/cos(36*PI/180))
      Node(3,4) = 1.0
      Node(1,5) = Node(1,3)+side*sin(18*PI/180)
      Node(2,5) = Node(2,3)+side*cos(18*PI/180)
      Node(3,5) = 0.0
      Node(1,6) = x0+0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(2,6) = Node(2,5)
      Node(3,6) = 1.0
      Node(1,7) = x0
      Node(2,7) = Node(2,5)+side*sin(36*PI/180)
      Node(3,7) = 0.0
      Node(1,8) = 2*x0 - Node(1,6)
      Node(2,8) = Node(2,6)
      Node(3,8) = 1.0
      Node(1,9) = 2*x0 - Node(1,5)
      Node(2,9) = Node(2,5)
      Node(3,9) = 0.0
      Node(1,10) = 2*x0 - Node(1,4)
      Node(2,10) = Node(2,4)
      Node(3,10) = 1.0

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3DR
      DPMaxCurves   = 1
      DPMaxPoints   = 10
      DPNumCurves   = 1
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'Surface Plot with Concave Edges'
      DPTitle(2)    = 'See {\iDefine Boundary} command '//
     1                'on {\iOptions} menu'
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 
		
      DocNum = DPlot_Plot8(DPVersion,dummy,Node,
     1           '[Caption("DPLOTLIB Surface Plot w/ Concave Edges")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[ContourLevels(20,0,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
c
c     At this point surface is convex - a pentagon. Add border
c     points, which in this example are simply a repeat of the x,y points
c     of the surface
c
      if(DocNum .gt. 0) then
        do i=1,10
          border(1,i) = Node(1,i)
          border(2,i) = Node(2,i)
        end do
        ret = DPlot_3DBorder(DocNum,10,border)
      endif
      return
      end
c
c======================================================================
c
      subroutine ThreeDScatter()
      implicit none
      include  'dplot.fi'
c
c     3D scatter plot
c
      integer*4 i, j, k
      integer*4 DocNum
      real*8    xmin, xmax, ymin, ymax, zmin, zmax
      real*8    xcenter, ycenter, zcenter
      real*8    phi, theta, theta_inc, radius
      real*8    dummy
      real*8    PI
      character szCommands*1024
      real*8    Node(3,3000)

      real*8   random
      external random

      PI = 4.*atan(1.0)

      DPVersion     = DPLOT_DDE_VERSION
      DPhwnd        = 0
      DPDataFormat  = DATA_3DS
      DPMaxCurves   = 3
      DPMaxPoints   = 1500
      DPNumCurves   = 3
      DPScale       = SCALE_LINEARX_LINEARY
      DPTitle(1)    = 'DPLOTLIB 3D Scatter Plot'
      DPXAxis       = 'x' 
      DPYAxis       = 'y' 

      DPNP(1) = 500
      DPNP(2) = 1000
      DPNP(3) = 1500
c
c     Keep track of the minimum and maximum X,Y, and Z values so we can
c     force plot extents to good values, "good" in this case being
c     whole numbers.
c
      xmin = 0
      xmax = 0
      ymin = 0
      ymax = 0
      zmin = 0
      zmax = 0

      k = 1       ! index into Scatter array
      do i = 1, DPNumCurves
c
c       Center of each sphere is at some random location between
c       (-5,-5,-5) and (5,5,5).
c
        xcenter = -5.0 + 10.0*random()
        ycenter = -5.0 + 10.0*random()
        zcenter = -5.0 + 10.0*random()
c
c       radii are proportional to number of points
c
        radius = 0.005 * DPNP(i)
c
        xmin = dmin1(xmin,xcenter-radius)
        xmax = dmax1(xmax,xcenter+radius)
        ymin = dmin1(ymin,ycenter-radius)
        ymax = dmax1(ymax,ycenter+radius)
        zmin = dmin1(zmin,zcenter-radius)
        zmax = dmax1(zmax,zcenter+radius)
c
c       increment in theta (angle around z axis) gives roughly
c       equal spacing between points in all directions:
c
        theta_inc = 2.0 * sqrt(dble(DPNP(i)))
        do j = 1, DPNP(i)
          phi = -PI / 2 + PI * dble(j-1) / dble(DPNP(i))
          theta = theta_inc * PI * dble(j-1) / dble(DPNP(i))
          Node(1,k) = xcenter + (radius * cos(theta) * cos(phi))
          Node(2,k) = ycenter + (radius * sin(theta) * cos(phi))
          Node(3,k) = zcenter + (radius * sin(phi))
          k = k + 1
        end do
      end do

      write(szCommands,'(A,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,A)')
     1           '[Caption("DPLOTLIB 3D Scatter Plot")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[TickInterval(1,1,1,1)][ContourAxes(1)]'//
     4           '[ColorBasis(1)][ContourColorScheme(2)]'//
     5           '[ContourCustomColors(255,65280,16711680)]'//
     6           '[ContourScales(1,1,1)]'//
     7           '[ZAxisLabel("z")]'//
     8           '[ManualScale(',
     9           int(xmin-0.5),int(ymin-0.5),
     A           int(xmax+0.5),int(ymax+0.5),
     B           int(zmin-0.5),int(zmax+0.5),
     C           ')][FontPoints(1,10)][FontPoints(6,10)]'//
     D           '[DocMaximize()]'//char(0)
      DocNum = DPlot_Plot8(DPVersion,dummy,Node,szCommands)
      return
      end
