c
c FTEST - tests DPLOTLIB.DLL with WATCOM Fortran 10.6
c
c This program has been tailored for WATCOM Fortran 10.6. Other Fortran compilers may require
c changes, particularly in the interface to DPlot_Plot.
c 
c
c Things to watch for with other compilers and general notes:
c
c 1) By default, most Fortran compilers translate the names of all functions/subroutines to UPPERCASE. 
c    The name "DPlot_Plot" in the DLL is case-specific. In other words you will most likely encounter
c    an "unresolved external" error on DPLOT_PLOT unless you take steps to tell the compiler not to
c    fold all characters to uppercase. If you are certain that you've taken the correct steps in this
c    regard and continue to get linker errors, the problem most likely lies with...
c
c 2) DPLOTLIB.LIB - The import library for DPLOTLIB.DLL. The Compaq Visual Fortran and Absoft Fortran
c    demos use the same import library as the C demo, which was produced as a byproduct of compiling
c    DPLOTLIB.DLL by the MSVC compiler. The DPLOTLIB.LIB used by the WATCOM Fortran demo was produced
c    with "wlib dplotlib +dplotlib.dll". This import library will almost certainly not work with other
c    compilers. If the import library does not work correctly, check your compiler's documentation for
c    the method used to produce import libraries from compiled DLL's.
c
c 3) This source code will work properly without changes with Open Source WATCOM Fortran 1.0. HOWEVER,
c    the supplied import library (DPLOTLIB.LIB) will NOT work with that compiler. To build FTEST with
c    Open Source WATCOM Fortran, first delete the import library DPLOTLIB.LIB, then build a new one
c    using the WATCOM library tool WLIB.EXE:
c     
c    wlib dplotlib +dplotlib.dll
c
c 4) The last parameter to DPlot_Plot is a character string containing DPlot commands. This character
c    string must be null-terminated (ends in a 0 byte). If you do NOT terminate this character string
c    with a char(0), at best DPlot will report an error in the command string; at worst DPlot and/or 
c    this demo will crash. This restriction does not apply to the character string members of the DPlot 
c    structure, although adding a terminating char(0) to those strings will not hurt anything.
c
c
c ALWAYS make sure to include this file:
c
c$include dplotapi.fi
c
c in any WATCOM FORTRAN routine that calls DPLOTLIB functions. Otherwise you'll most likely
c get 'unresolved external' errors when linking.
c
      program ftest
      implicit none
      include  'dplot.fi'

      integer option

 100  continue

      write(*,*) 'Enter 0 to exit'
      write(*,*) '      1 for XY plot example'
      write(*,*) '      2 for contour plot of gridded data'
      write(*,*) '      3 for contour plot of randomly-spaced points'
      write(*,*) '      4 for box-and-whisker example'
      write(*,*) '      5 for surface plot with concave border'
      write(*,*) '      6 for a 3D scatter plot'
      write(*,*) '? '
      read(*,*) option

      select case(option)
      case(1)
        call XYExample()

      case(2)
        call ContourPlot1()

      case(3)
        call ContourPlot2()

      case(4)
        call BoxAndWhiskerPlot()

      case(5)
        call ConcaveBorderExample()

      case(6)
        call ThreeDScatter()

      case(0)
        goto 9999

      end select

      goto 100
9999  continue
      end
c
c=======================================================================
c
      real*8 function random()
      implicit none
      external urand
      real     urand
      integer  seed /75347/
      random = urand(seed)
      return
      end
c
c=======================================================================
c
      subroutine XYExample()
      implicit none
      include  'dplot.fi'
c
c     Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
c     In this case we've used DATA_XYYY (one X array for one or more Y arrays).
c
c     Since the X values are evenly spaced we could also use
c     DATA_DXY  - X has only 2 elements, DX and X0
c
c     And you can ALWAYS use
c     DATA_XYXY - Each curve uses its own X array. This is the only
c                 option available if the curves have different X values or
c                 a different number of points.

      integer*4 NP
      parameter (NP=1001)
      record /DPLOT/ DPlot
      integer*4 i
      integer*4 ret
      real*8    x(NP), y(2*NP)
      real*8    PI
      ret = 1
      PI = 4.*atan(1.0)
      do i=1,NP
        x(i)   = (4.*i)/(NP-1)
        y(i)   = sin(PI*x(i))
        y(i+NP)= cos(PI*x(i))
      end do

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_XYYY
      DPlot.MaxCurves   = 2
      DPlot.MaxPoints   = NP
      DPlot.NumCurves   = 2
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.LegendX     = 0.05
      DPlot.LegendY     = 0.05
      DPlot.NP(1)       = NP
      DPlot.NP(2)       = NP
      DPlot.LineType(1) = LINESTYLE_SOLID
      DPlot.LineType(2) = LINESTYLE_LONGDASH
      DPlot.Legend(1)   = 'sin({\sp}x)'
c                              {\s is DPlot-speak for "use symbol font"
      DPlot.Legend(2)   = 'cos({\sp}x)'  
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
c		
c     The command string can be as complex as you like, limited to 32K characters.
c     As an alternative, if your program will be producing many similar plots then
c     you might prefer to create a preferences file, edit the file with a text 
c     editor to remove unwanted entries, and use the [GetPreferences("filename")]
c     command. 
c
      ret = DPlot_Plot8(DPlot,x,y,
     1                 '[ManualScale(0,-1.25,4,1.25)]'//
     2                 '[TickInterval(1,0.5,0.25)]'//
     3                 '[Caption("DPLOTLIB XY Test")]'//
     4                 '[DocMaximize()]'//
     5                 '[ClearEditFlag()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ContourPlot1()
      implicit none
      include  'dplot.fi'
c
c     Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
c
      integer*4 NX, NY
      parameter (NX=61, NY=41)
      record  /DPLOT/ DPlot
      integer*4 i, j, ret
      real*8    extents(4)
      real*8    xx, yy
      real*8    z(NX*NY)

c     extents are, in order, xlo, ylo, xhi, yhi
      extents(1) = -3.
      extents(2) = -2.
      extents(3) =  3.
      extents(4) =  2.
      do i=1,Nx
        xx = extents(1) + ((extents(3)-extents(1))*i)/(Nx-1)
        do j=1,Ny
          yy = extents(2) + ((extents(4)-extents(2))*j)/(Ny-1)
          z((i-1)*Ny+j) = sin(xx)*cos(yy)
        end do
      end do

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3D
c
c     MaxCurves and MaxPoints members are used for the size of the grid, in grid cells
c
      DPlot.MaxCurves   = Nx-1
      DPlot.MaxPoints   = Ny-1
      DPlot.NumCurves   = 1
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPlot.Title(2)    = 'z = sin(x)*cos(y)'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
		
      ret = DPlot_Plot8(DPlot,extents,z,
     1       '[Caption("DPLOTLIB Contour Test 1")]'//
     2       '[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]'//
     3       '[ContourView(30,20)][ContourLevels(20,-1,1)]'//
     4       '[ContourScales(1,1,1)]'//
     5       '[ContourLighting(2,0.2)]'//                     ! Phong shading, 20% ambient light
     6       '[FontPoints(1,10)][FontPoints(6,10)]'//
     7       '[FontPoints(4,12)][FontPoints(5,12)][FontPoints(8,12)]'//
     8       '[ZAxisLabel("z")][DocMaximize()]'//char(0))
      return
	  end
c
c======================================================================
c
      subroutine ContourPlot2()
      implicit none
      include  'dplot.fi'
c
c     Contour plot with randomly spaced points
c
      structure /XYZ/
        real*8 x
        real*8 y
        real*8 z
      end structure

      integer*4 i, ret
      real*8    PI
      real*8    dummy

      integer*4 NZ
      parameter (NZ=1000)
      record  /XYZ/ Node(NZ)
      record  /DPLOT/ DPlot

      real*8   random
      external random

      PI = 4.*atan(1.0)
      do i=1,Nz
         Node(i).x = -PI + 2*PI*random()
         Node(i).y = -PI + 2*PI*random()
         Node(i).z = sin(Node(i).x)*cos(Node(i).y)
      end do

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3DR
      DPlot.MaxCurves   = 1
      DPlot.MaxPoints   = Nz
      DPlot.NumCurves   = 1
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL' 
      DPlot.Title(2)    = 'z = sin(x)*cos(y)'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
		
      ret = DPlot_Plot8(DPlot,dummy,Node,
     1           '[Caption("DPLOTLIB Contour Test 2")]'//
     2           '[Contour3D(0)][ContourGrid(0)]'//
     3           '[ContourLevels(21,-1,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine BoxAndWhiskerPlot()
      implicit none
      include  'dplot.fi'
c
c     Box-and-whisker plot - 3 groups
c
      integer*4 i, j, k, ret
      real*8    dummy

      integer*4 NP
      parameter (NP=150)
      record  /DPLOT/ DPlot
      real*8  y(NP)

      real*8   random
      external random

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_1D
      DPlot.MaxCurves   = 3
      DPlot.MaxPoints   = 50
      DPlot.NumCurves   = 3
      DPlot.LegendX		= 0.02
      DPlot.LegendY		= 0.7
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Data sent to DPlot via DPLOTLIB.DLL'
      DPlot.XAxis       = ' '
      DPlot.YAxis       = 'Amplitude'

      do i=1,DPlot.NumCurves
        DPlot.NP(i) = DPlot.MaxPoints
        DPlot.Legend(i) = 'Group '//char(64+i)//char(0)
      end do

c     Generate group data, centered around 5.0 with progressively larger variations
      k = 1
      do i=1,DPlot.NumCurves
        do j=1,DPlot.NP(i)
          y(k) = 5.0 + (i+1)*(-0.5+random())
          k = k+1
        end do
      end do
		
      ret = DPlot_Plot8(DPlot,dummy,y,
     1           '[Caption("DPLOTLIB Box-and-whisker Test")]'//
     2           '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]'//
     3           '[Stat_BWOptions(1,1,0,1,1.5,2,3)]'//
     4           '[Stat_GrandMean(3,30)]'//
     5           '[FontPoints(1,10)][FontPoints(6,10)]'//
     6           '[DocMaximize()]'//char(0))
      return
      end
c
c======================================================================
c
      subroutine ConcaveBorderExample()
      implicit none
      include  'dplot.fi'
c
c     Surface plot with concave border
c
      structure /XYZ/
        real*8 x
        real*8 y
        real*8 z
      end structure

      structure /XYPT/
        real*8 x
        real*8 y
      end structure

      integer*4 i, DocNum, ret
      real*8    x0, y0, side
      real*8    dummy
      real*8    PI
      record /XYZ/ Node(10)
      record /XYPT/ Border(10)
      record /DPLOT/ DPlot


      PI = 4.*atan(1.0)

      x0 = 0.             ! center of star/pentagon
      y0 = 0.
      side = 1.0          ! length of each pentagon side

      Node(1).x = x0 - 0.5*side
      Node(1).y = y0 - 0.5*side*tan(54*PI/180)
      Node(1).z = 0.0
      Node(2).x = x0
      Node(2).y = Node(1).y+0.5*side*tan(36*PI/180)
      Node(2).z = 1.0
      Node(3).x = x0 + 0.5*side
      Node(3).y = Node(1).y
      Node(3).z = 0.0
      Node(4).x = Node(3).x-0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(4).y = Node(3).y+0.5*side*(cos(18*PI/180)/cos(36*PI/180))
      Node(4).z = 1.0
      Node(5).x = Node(3).x+side*sin(18*PI/180)
      Node(5).y = Node(3).y+side*cos(18*PI/180)
      Node(5).z = 0.0
      Node(6).x = x0+0.5*side*(sin(18*PI/180)/cos(36*PI/180))
      Node(6).y = Node(5).y
      Node(6).z = 1.0
      Node(7).x = x0
      Node(7).y = Node(5).y+side*sin(36*PI/180)
      Node(7).z = 0.0
      Node(8).x = 2*x0 - Node(6).x
      Node(8).y = Node(6).y
      Node(8).z = 1.0
      Node(9).x = 2*x0 - Node(5).x
      Node(9).y = Node(5).y
      Node(9).z = 0.0
      Node(10).x = 2*x0 - Node(4).x
      Node(10).y = Node(4).y
      Node(10).z = 1.0

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3DR
      DPlot.MaxCurves   = 1
      DPlot.MaxPoints   = 10
      DPlot.NumCurves   = 1
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'Surface Plot with Concave Edges'
      DPlot.Title(2)    = 'See {\iDefine Boundary} command '//
     1                    'on {\iOptions} menu'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 
		
      DocNum = DPlot_Plot8(DPlot,dummy,Node,
     1           '[Caption("DPLOTLIB Surface Plot w/ Concave Edges")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[ContourLevels(20,0,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0))
c
c     At this point surface is convex - a pentagon. Add border
c     points, which in this example are simply a repeat of the x,y points
c     of the surface
c
      if(DocNum .gt. 0) then
        do i=1,10
          border(i).x = Node(i).x
          border(i).y = Node(i).y
        end do
        ret = DPlot_3DBorder(DocNum,10,border)
      endif
      return
      end
c
c======================================================================
c
      subroutine ThreeDScatter()
      implicit none
      include  'dplot.fi'
c
c     3D scatter plot
c
      structure /XYZ/
        real*8 x
        real*8 y
        real*8 z
      end structure

      integer*4 i, j, k
      integer*4 DocNum
      real*8    xmin, xmax, ymin, ymax, zmin, zmax
      real*8    xcenter, ycenter, zcenter
      real*8    phi, theta, theta_inc, radius
      real*8    dummy
      real*8    PI
      character szCommands*1024
      record /XYZ/ Node(3000)
      record /DPLOT/ DPlot

      real*8   random
      external random

      PI = 4.*atan(1.0)

      DPlot.Version     = DPLOT_DDE_VERSION
      DPlot.hwnd        = 0
      DPlot.DataFormat  = DATA_3DS
      DPlot.MaxCurves   = 3
      DPlot.MaxPoints   = 1500
      DPlot.NumCurves   = 3
      DPlot.Scale       = SCALE_LINEARX_LINEARY
      DPlot.Title(1)    = 'DPLOTLIB 3D Scatter Plot'
      DPlot.XAxis       = 'x' 
      DPlot.YAxis       = 'y' 

      DPlot.NP(1) = 500
      DPlot.NP(2) = 1000
      DPlot.NP(3) = 1500
c
c     Keep track of the minimum and maximum X,Y, and Z values so we can
c     force plot extents to good values, "good" in this case being
c     whole numbers.
c
      xmin = xmax = 0
      ymin = ymax = 0
      zmin = zmax = 0

      k = 1       ! index into Scatter array
      do i = 1, DPlot.NumCurves
c
c       Center of each sphere is at some random location between
c       (-5,-5,-5) and (5,5,5).
c
        xcenter = -5.0 + 10.0*random()
        ycenter = -5.0 + 10.0*random()
        zcenter = -5.0 + 10.0*random()
c
c       radii are proportional to number of points
c
        radius = 0.005 * DPlot.NP(i)
c
        xmin = dmin1(xmin,xcenter-radius)
        xmax = dmax1(xmax,xcenter+radius)
        ymin = dmin1(ymin,ycenter-radius)
        ymax = dmax1(ymax,ycenter+radius)
        zmin = dmin1(zmin,zcenter-radius)
        zmax = dmax1(zmax,zcenter+radius)
c
c       increment in theta (angle around z axis) gives roughly
c       equal spacing between points in all directions:
c
        theta_inc = 2.0 * sqrt(dble(DPlot.NP(i)))
        do j = 1, DPlot.NP(i)
          phi = -PI / 2 + PI * dble(j-1) / dble(DPlot.NP(i))
          theta = theta_inc * PI * dble(j-1) / dble(DPlot.NP(i))
          Node(k).x = xcenter + (radius * cos(theta) * cos(phi))
          Node(k).y = ycenter + (radius * sin(theta) * cos(phi))
          Node(k).z = zcenter + (radius * sin(phi))
          k = k + 1
        end do
      end do

      write(szCommands,'(A,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,1H,,I3,A)')
     1           '[Caption("DPLOTLIB 3D Scatter Plot")]'//
     2           '[Contour3D(0)][ContourMethod(0)][ContourGrid(0)]'//
     3           '[TickInterval(1,1,1,1)][ContourAxes(1)]'//
     4           '[ColorBasis(1)][ContourColorScheme(2)]'//
     5           '[ContourCustomColors(255,65280,16711680)]'//
     6           '[ContourScales(1,1,1)]'//
     7           '[ZAxisLabel("z")]'//
     8           '[ManualScale(',
     9           int(xmin-0.5),int(ymin-0.5),
     A           int(xmax+0.5),int(ymax+0.5),
     B           int(zmin-0.5),int(zmax+0.5),
     C           ')][FontPoints(1,10)][FontPoints(6,10)]'//
     D           '[DocMaximize()]'//char(0)
      DocNum = DPlot_Plot8(DPlot,dummy,Node,szCommands)
      return
      end
