c g77 notes
c
c With the exception of the standard caveats of the switches used to call c
c programs, this program follows the fortran 77 standard.  Use this program
c if the compiler does not understand the VAX structure statements and you 
c cant use fortran 90/95.
c
c One place the program doesnt follow the standard is in the include file, 
c dplot.g77.  the DPLOT structure is created using a common block.  The 
c standard doesnt allow mixing of character, reals, and integers in a 
c common block. I dont know of any compilers that dont support this extension,
c so it shouldnt cause any problems.
C
C g77 uses the _cdecl calling convention by default, while all DPLOTLIB.DLL
c functions are _stdcall. There are two workarounds for this:
c
c 1) Using standard _stdcall version of DPLOTLIB.DLL:
c
c    Compile with
c    g77 -c -fno-backslash -fcase-preserve -fno-underscoring -mrtd ftest.f
c    g77 -o ftest ftest.o dplotlib.a
c
c    The downside to doing this occurs if your own program calls functions in
c    some OTHER dll that use the _cdecl calling convention. In that case,
c    you'll need to instead use:
c
c 2) Using _cdecl version of DPLOTLIB.DLL:
c
c    Compile with: 
c
c    g77 -c -fno-backslash -fcase-preserve -fno-underscoring ftest.f
c    g77 -o ftest ftest.o dplotlib_cc.a
c
c    and make sure to include DPLOTLIB_CC.DLL with your project rather than
c    DPLOTLIB.DLL
C
C end g77 notes
c
c FTEST - tests DPLOTLIB.DLL in Fortran.
c
      program ftest
      implicit none
      include  'dplot.g77'
c
C Setup array limits required in fortran 77 (g77 compiler)
      integer pMAX, xMAX, yMAX, zMAX
      parameter (pMAX=1000)
      parameter (xMAX=1001)
      parameter (yMAX=xMAX*2)
      parameter (zMAX=1000 )
      character*32000 cString
C
      real*4  x(xMAX),    y(yMAX),    z(zMAX)
      real*4  XYZ(3,pMAX)

      integer  DPlot_Plot
      external DPlot_Plot
      integer  DPlot_AddData
      external DPlot_AddData
      integer  DPlot_Command
      external DPlot_Command
      real*4   random

      real*4 NULL_POINTER(1)
      real*4  xx, yy
      real*4  extents(4)
      real*4  PI
      integer option
      integer DocNum
      integer i, j, k
      integer NP                !  Number of points we'll plot in XY example
      integer Nz                !  Number of points we'll plot in randomly-spaced XYZ points example
      integer Nx, Ny            !  Size of grid for contour plot
      integer Nyw               !  for whisker plots
      parameter (Nx=61)
      parameter (Ny=41)
      integer ret
      integer seed

      PI = 4.*atan(1.0)

 100  continue

        write(*,*) 'Enter 0 to exit'
        write(*,*) '      1 for XY plot example'
        write(*,*) '      2 for contour plot of gridded data'
        write(*,*) '      3 for contour plot of randomly-spaced points'
        write(*,*) '      4 for box-and-whisker example'
        write(*,*) '? '
        read(*,*) option

        select case(option)
        case(1)
c       Plot sin(PI*x) and cos(PI*x) from x = 0 to 4.
c       In this case we ve used DATA_XYYY (one X array for one or more Y arrays).
c
c       Since the X values are evenly spaced we could also use
c       DATA_DXY  - X has only 2 elements, DX and X0
c
c       And you can ALWAYS use
c       DATA_XYXY - Each curve uses its own X array. This is the only
c                   option available if the curves have different X values or
c                   a different number of points.
        NP = 1001
        do i=1,NP
          x(i)   = (4.*(i-1))/(NP-1)
          y(i)   = sin(PI*x(i))
          y(i+NP)= cos(PI*x(i))
        end do

        DPVersion       = DPLOT_DDE_VERSION
        DPhwnd          = 0
        DPDataFormat    = DATA_XYYY
        DPMaxCurves     = 2             ! Must be >= number of curves we plot
        DPMaxPoints     = NP            ! Anything >= NP will do
        DPNumCurves     = 2
        DPScale         = SCALE_LINEARX_LINEARY
        DPLegendX       = 0.05
        DPLegendY       = 0.05
        DPNP(1)         = NP
        DPNP(2)         = NP
        DPLineType(1)   = LINESTYLE_SOLID
        DPLineType(2)   = LINESTYLE_LONGDASH
        DPLegend(1)     = 'sin({\sp}x)'//char(0)     ! {\s is DPlot-speak for "use symbol font"
        DPLegend(2)     = 'cos({\sp}x)'//char(0)  
        DPTitle(1)      = 'Data sent to DPlot via DPLOTLIB.DLL'//char(0)
        DPXAxis         = 'x'//char(0) 
        DPYAxis         = 'y'//char(0) 
c
c       The command string can be as complex as you like, limited to 32K characters.
c       As an alternative, if your program will be producing many similar plots then
c       you might prefer to create a preferences file, edit the file with a text 
c       editor to remove unwanted entries, and use the [GetPreferences("filename")]
c       command. 
c
        cString = '[ManualScale(0,-1.25,4,1.25)]'//
     1            '[TickInterval(1,0.5,0.25)]'//
     2            '[Caption("DPLOTLIB XY Test")]'//
     3            '[DocMaximize()]'//
     4            '[ClearEditFlag()]'//char(0) 
        DocNum = DPlot_Plot( DPVersion, x, y, cString )
 
        case(2)
c
c       Contour plot of Z values on a rectangular grid.  z = sin(x)*cos(y)
c
          extents(1) = -3.                ! xlo
          extents(2) = -2.                ! ylo
          extents(3) =  3.                ! xhi
          extents(4) =  2.                ! yhi
          do i=1,Nx
            xx = extents(1) + ((extents(3)-extents(1))*i)/(Nx-1)
            do j=1,Ny
              yy = extents(2) + ((extents(4)-extents(2))*j)/(Ny-1)
              z((i-1)*Ny+j) = sin(xx)*cos(yy)
            end do
          end do
  
          DPVersion       = DPLOT_DDE_VERSION
          DPhwnd          = 0
          DPDataFormat    = DATA_3D
          DPMaxCurves     = Nx-1          ! Size of the grid, in grid cells
          DPMaxPoints     = Ny-1
          DPNumCurves     = 1
          DPScale         = SCALE_LINEARX_LINEARY
          DPTitle(1)      = 'Data sent to DPlot via DPLOTLIB.DLL'//
     1                       char(0)
          DPTitle(2)      = 'z = sin(x)*cos(y)'//char(0)
          DPXAxis         = 'x'//char(0) 
          DPYAxis         = 'y'//char(0) 

          cString = '[Caption("DPLOTLIB Contour Test 1")]'//
     1           '[Contour3D(1)][ContourGrid(1)][ContourAxes(1)]'//
     2           '[ContourView(30,20)][ContourLevels(20,-1,1)]'//
     3           '[ContourScales(1,1,1)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[ZAxisLabel("z")][DocMaximize()]'//char(0)
                  
          ret = DPlot_Plot(DPVersion,extents,z,cString)
  
        case(3)
c
c       Contour plot with randomly spaced points
c
          Nz = pMAX
          seed = 1
          do i=1,Nz
             XYZ(1,i) = -PI + 2*PI*random(seed)
             XYZ(2,i) = -PI + 2*PI*random(seed)
             XYZ(3,i) = sin(XYZ(1,i))*cos(XYZ(2,i))
          end do
  
          DPVersion       = DPLOT_DDE_VERSION
          DPhwnd          = 0
          DPDataFormat    = DATA_3DR
          DPMaxCurves     = 1
          DPMaxPoints     = Nz
          DPNumCurves     = 1
          DPScale         = SCALE_LINEARX_LINEARY
          DPTitle(1)      = 'Data sent to DPlot via DPLOTLIB.DLL'//
     1                      char(0)
          DPTitle(2)      = 'z = sin(x)*cos(y)'//char(0)
          DPXAxis         = 'x'//char(0) 
          DPYAxis         = 'y'//char(0) 

          cString = '[Caption("DPLOTLIB Contour Test 2")]'//
     1           '[Contour3D(0)][ContourGrid(0)]'//
     2           '[ContourLevels(21,-1,1)]'//
     3           '[FontPoints(1,10)][FontPoints(6,10)]'//
     4           '[DocMaximize()]'//char(0)
          ret = DPlot_Plot(DPVersion,NULL_POINTER,XYZ,cString)
  
        case(4)
c
c       Box-and-whisker plot - 3 groups
c
          DPVersion           = DPLOT_DDE_VERSION
          DPhwnd              = 0
          DPDataFormat        = DATA_1D
          DPMaxCurves         = 3
          DPMaxPoints         = 80
          DPNumCurves         = 3
          DPScale             = SCALE_LINEARX_LINEARY
          DPLegendX           = 0.02
          DPLegendY           = 0.7
          DPTitle(1)          = 'Data sent to DPlot via DPLOTLIB.DLL'//
     1                           char(0)
          DPYAxis             = 'Amplitude'//char(0)
  
c       Number of points in each group is 50 +/- 15:
          Nyw = 0
          do i=1,DPNumCurves
            DPNP(i) = 50 + nint(30*(-.5+random(seed)))
            Nyw = Nyw + DPNP(i)
            DPLegend(i) = 'Group '//char(64+i)//char(0)
          end do
  
c       Generate group data, centered around 5.0 with progressively larger variations
          k = 1
          do i=1,DPNumCurves
            do j=1,DPNP(i)
              y(k) = 5.0 + (i+1)*(-0.5+random(seed))
              k = k+1
            end do
          end do

          cString = '[Caption("DPLOTLIB Box-and-whisker Test")]'//
     1           '[Stat_PlotType(0)][Stat_GroupInfo(15,4)]'//
     2           '[Stat_BWOptions(1,1,0,1,1.5,2,3)]'//
     3           '[Stat_GrandMean(3,30)]'//
     4           '[FontPoints(1,10)][FontPoints(6,10)]'//
     5           '[DocMaximize()]'//char(0)
          ret = DPlot_Plot(DPVersion,NULL_POINTER,y,cString)
  
        case(0)
          goto 9999
  
        end select

      goto 100
9999  continue
      end
c
c=======================================================================
c
      real*4 function random(iseed)
      implicit none
      integer*4 iseed

      iseed = mod(iseed*7141+54773,259200)
      random = real(iseed)/259200.
      return
      end
